from _typeshed import Incomplete
from logging import Logger

from authlib.oidc.core import OpenIDImplicitGrant

log: Logger

class OpenIDHybridGrant(OpenIDImplicitGrant):
    AUTHORIZATION_CODE_LENGTH: int
    RESPONSE_TYPES: Incomplete
    GRANT_TYPE: str
    DEFAULT_RESPONSE_MODE: str
    def generate_authorization_code(self): ...
    def save_authorization_code(self, code, request) -> None: ...
    def validate_authorization_request(self): ...
    def create_granted_params(self, grant_user): ...



from _typeshed import Incomplete

from authlib.jose import JWTClaims
from authlib.oauth2.rfc6749 import TokenMixin
from authlib.oauth2.rfc6750 import BearerTokenValidator

logger: Incomplete

class JWTBearerToken(TokenMixin, JWTClaims):
    def check_client(self, client): ...
    def get_scope(self): ...
    def get_expires_in(self): ...
    def is_expired(self): ...
    def is_revoked(self): ...

class JWTBearerTokenValidator(BearerTokenValidator):
    TOKEN_TYPE: str
    token_cls = JWTBearerToken
    public_key: Incomplete
    claims_options: Incomplete
    def __init__(self, public_key, issuer=None, realm=None, **extra_attributes) -> None: ...
    def authenticate_token(self, token_string): ...



from typing import Any
from typing_extensions import TypeAlias

_Image: TypeAlias = Any  # actually PIL.Image, but not worth adding a dependency

def shift_hue(image: _Image, hue: float) -> _Image: ...
def make_transparent(image: _Image) -> _Image: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import ClassVar, Final, overload

from docutils import nodes
from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class PropagateTargets(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class AnonymousHyperlinks(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class IndirectHyperlinks(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...
    def resolve_indirect_target(self, target: nodes.Element) -> None: ...
    def nonexistent_indirect_target(self, target: nodes.Element) -> None: ...
    def circular_indirect_reference(self, target: nodes.Element) -> None: ...
    def indirect_target_error(self, target: nodes.Element, explanation) -> None: ...
    def resolve_indirect_references(self, target: nodes.Element) -> None: ...

class ExternalTargets(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class InternalTargets(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...
    def resolve_reference_ids(self, target: nodes.Element) -> None: ...

class Footnotes(Transform):
    default_priority: ClassVar[int]
    autofootnote_labels: list[str] | None
    symbols: ClassVar[list[str]]
    def apply(self) -> None: ...
    def number_footnotes(self, startnum: int) -> int: ...
    def number_footnote_references(self, startnum: int) -> None: ...
    def symbolize_footnotes(self) -> None: ...
    def resolve_footnotes_and_citations(self) -> None: ...
    @overload
    def resolve_references(self, note: nodes.footnote, reflist: Iterable[nodes.footnote_reference]) -> None: ...
    @overload
    def resolve_references(self, note: nodes.citation, reflist: Iterable[nodes.citation_reference]) -> None: ...
    @overload
    def resolve_references(self, note: nodes.title, reflist: Iterable[nodes.title_reference]) -> None: ...

class CircularSubstitutionDefinitionError(Exception): ...

class Substitutions(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class TargetNotes(Transform):
    default_priority: ClassVar[int]
    classes: Incomplete
    def __init__(self, document: nodes.document, startnode: nodes.Node) -> None: ...
    def apply(self) -> None: ...
    def make_target_footnote(self, refuri: str, refs: list[Incomplete], notes: dict[Incomplete, Incomplete]): ...

class CitationReferences(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class DanglingReferences(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class DanglingReferencesVisitor(nodes.SparseNodeVisitor):
    document: nodes.document
    def __init__(self, document: nodes.document, unknown_reference_resolvers) -> None: ...
    def unknown_visit(self, node: nodes.Node) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...



from _typeshed import Incomplete
from datetime import datetime
from decimal import Decimal
from typing import Final

from braintree.add_on import AddOn
from braintree.address import Address
from braintree.amex_express_checkout_card import AmexExpressCheckoutCard
from braintree.android_pay_card import AndroidPayCard
from braintree.apple_pay_card import ApplePayCard
from braintree.authorization_adjustment import AuthorizationAdjustment
from braintree.credit_card import CreditCard
from braintree.customer import Customer
from braintree.descriptor import Descriptor
from braintree.disbursement_detail import DisbursementDetail
from braintree.discount import Discount
from braintree.dispute import Dispute
from braintree.europe_bank_account import EuropeBankAccount
from braintree.facilitated_details import FacilitatedDetails
from braintree.facilitator_details import FacilitatorDetails
from braintree.local_payment import LocalPayment
from braintree.masterpass_card import MasterpassCard
from braintree.meta_checkout_card import MetaCheckoutCard
from braintree.meta_checkout_token import MetaCheckoutToken
from braintree.package_details import PackageDetails
from braintree.payment_facilitator import PaymentFacilitator
from braintree.paypal_account import PayPalAccount
from braintree.paypal_here import PayPalHere
from braintree.resource import Resource
from braintree.resource_collection import ResourceCollection
from braintree.risk_data import RiskData
from braintree.samsung_pay_card import SamsungPayCard
from braintree.sepa_direct_debit_account import SepaDirectDebitAccount
from braintree.status_event import StatusEvent
from braintree.subscription_details import SubscriptionDetails
from braintree.three_d_secure_info import ThreeDSecureInfo
from braintree.transfer import Transfer
from braintree.us_bank_account import UsBankAccount
from braintree.venmo_account import VenmoAccount
from braintree.visa_checkout_card import VisaCheckoutCard

class Transaction(Resource):
    class CreatedUsing:
        FullInformation: Final = "full_information"
        Token: Final = "token"

    class GatewayRejectionReason:
        ApplicationIncomplete: Final = "application_incomplete"
        Avs: Final = "avs"
        AvsAndCvv: Final = "avs_and_cvv"
        Cvv: Final = "cvv"
        Duplicate: Final = "duplicate"
        ExcessiveRetry: Final = "excessive_retry"
        Fraud: Final = "fraud"
        RiskThreshold: Final = "risk_threshold"
        ThreeDSecure: Final = "three_d_secure"
        TokenIssuance: Final = "token_issuance"

    class ReasonCode:
        ANY_REASON_CODE: Final = "any_reason_code"

    class Source:
        Api: Final = "api"
        ControlPanel: Final = "control_panel"
        Recurring: Final = "recurring"

    class Status:
        AuthorizationExpired: Final = "authorization_expired"
        Authorized: Final = "authorized"
        Authorizing: Final = "authorizing"
        Failed: Final = "failed"
        GatewayRejected: Final = "gateway_rejected"
        ProcessorDeclined: Final = "processor_declined"
        Settled: Final = "settled"
        SettlementConfirmed: Final = "settlement_confirmed"
        SettlementDeclined: Final = "settlement_declined"
        SettlementFailed: Final = "settlement_failed"
        SettlementPending: Final = "settlement_pending"
        Settling: Final = "settling"
        SubmittedForSettlement: Final = "submitted_for_settlement"
        Voided: Final = "voided"

    class Type:
        Credit: Final = "credit"
        Sale: Final = "sale"

    class IndustryType:
        Lodging: Final = "lodging"
        TravelAndCruise: Final = "travel_cruise"
        TravelAndFlight: Final = "travel_flight"

    class AdditionalCharge:
        Restaurant: Final = "restaurant"
        GiftShop: Final = "gift_shop"
        MiniBar: Final = "mini_bar"
        Telephone: Final = "telephone"
        Laundry: Final = "laundry"
        Other: Final = "other"

    @staticmethod
    def adjust_authorization(transaction_id, amount): ...
    @staticmethod
    def clone_transaction(transaction_id, params): ...
    @staticmethod
    def credit(params=None): ...
    @staticmethod
    def find(transaction_id: str) -> Transaction: ...
    @staticmethod
    def refund(transaction_id, amount_or_options=None): ...
    @staticmethod
    def sale(params=None): ...
    @staticmethod
    def search(*query) -> ResourceCollection: ...
    @staticmethod
    def submit_for_settlement(transaction_id, amount=None, params=None): ...
    @staticmethod
    def update_details(transaction_id, params=None): ...
    @staticmethod
    def void(transaction_id): ...
    @staticmethod
    def create(params): ...
    @staticmethod
    def clone_signature(): ...
    @staticmethod
    def create_signature(): ...
    @staticmethod
    def submit_for_settlement_signature(): ...
    @staticmethod
    def submit_for_partial_settlement_signature(): ...
    @staticmethod
    def package_tracking_signature(): ...
    @staticmethod
    def package_tracking(transaction_id, params=None): ...
    @staticmethod
    def update_details_signature(): ...
    @staticmethod
    def refund_signature(): ...
    @staticmethod
    def submit_for_partial_settlement(transaction_id, amount, params=None): ...
    amount: Decimal
    tax_amount: Decimal | None
    discount_amount: Decimal | None
    shipping_amount: Decimal | None
    billing_details: Address
    credit_card_details: CreditCard
    packages: list[PackageDetails]
    paypal_details: PayPalAccount
    paypal_here_details: PayPalHere
    local_payment_details: LocalPayment
    sepa_direct_debit_account_details: SepaDirectDebitAccount
    europe_bank_account_details: EuropeBankAccount
    us_bank_account: UsBankAccount
    apple_pay_details: ApplePayCard
    android_pay_card_details: AndroidPayCard
    amex_express_checkout_card_details: AmexExpressCheckoutCard
    venmo_account_details: VenmoAccount
    visa_checkout_card_details: VisaCheckoutCard
    masterpass_card_details: MasterpassCard
    samsung_pay_card_details: SamsungPayCard
    meta_checkout_card_details: MetaCheckoutCard
    meta_checkout_token_details: MetaCheckoutToken
    sca_exemption_requested: Incomplete
    customer_details: Customer
    shipping_details: Address
    add_ons: list[AddOn]
    discounts: list[Discount]
    status_history: list[StatusEvent]
    subscription_details: SubscriptionDetails
    descriptor: Descriptor
    disbursement_details: DisbursementDetail
    disputes: list[Dispute]
    authorization_adjustments: list[AuthorizationAdjustment]
    payment_instrument_type: Incomplete
    risk_data: RiskData | None
    three_d_secure_info: ThreeDSecureInfo | None
    facilitated_details: FacilitatedDetails
    facilitator_details: FacilitatorDetails
    network_transaction_id: Incomplete
    payment_facilitator: PaymentFacilitator
    transfer: Transfer
    subscription_id: str
    created_at: datetime
    def __init__(self, gateway, attributes) -> None: ...
    @property
    def vault_billing_address(self): ...
    @property
    def vault_credit_card(self): ...
    @property
    def vault_customer(self): ...
    @property
    def is_disbursed(self): ...
    @property
    def line_items(self): ...



from markdown.core import Markdown

from . import util

def build_preprocessors(md: Markdown, **kwargs) -> util.Registry[Preprocessor]: ...

class Preprocessor(util.Processor):
    def run(self, lines: list[str]) -> list[str]: ...

class NormalizeWhitespace(Preprocessor): ...
class HtmlBlockPreprocessor(Preprocessor): ...



from _typeshed import Incomplete

from braintree.amex_express_checkout_card import AmexExpressCheckoutCard
from braintree.android_pay_card import AndroidPayCard
from braintree.apple_pay_card import ApplePayCard
from braintree.credit_card import CreditCard
from braintree.error_result import ErrorResult
from braintree.europe_bank_account import EuropeBankAccount
from braintree.masterpass_card import MasterpassCard
from braintree.paypal_account import PayPalAccount
from braintree.resource import Resource
from braintree.samsung_pay_card import SamsungPayCard
from braintree.sepa_direct_debit_account import SepaDirectDebitAccount
from braintree.successful_result import SuccessfulResult
from braintree.unknown_payment_method import UnknownPaymentMethod
from braintree.us_bank_account import UsBankAccount
from braintree.venmo_account import VenmoAccount
from braintree.visa_checkout_card import VisaCheckoutCard

class PaymentMethod(Resource):
    @staticmethod
    def create(params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult: ...
    @staticmethod
    def find(
        payment_method_token: str,
    ) -> (
        AndroidPayCard
        | ApplePayCard
        | EuropeBankAccount
        | CreditCard
        | PayPalAccount
        | UsBankAccount
        | VenmoAccount
        | VisaCheckoutCard
        | AmexExpressCheckoutCard
        | SepaDirectDebitAccount
        | MasterpassCard
        | SamsungPayCard
        | UnknownPaymentMethod
    ): ...
    @staticmethod
    def update(payment_method_token: str, params) -> SuccessfulResult | ErrorResult: ...
    @staticmethod
    def delete(payment_method_token: str, options=None) -> SuccessfulResult: ...
    @staticmethod
    def create_signature() -> (
        list[
            str
            | dict[str, list[str | dict[str, list[str]]]]
            | dict[str, list[str | dict[str, list[str]] | dict[str, list[str | dict[str, list[str | dict[str, list[str]]]]]]]]
            | dict[str, list[str]]
        ]
    ): ...
    @staticmethod
    def signature(
        type: str,
    ) -> list[
        str
        | dict[str, list[str | dict[str, list[str]]]]
        | dict[str, list[str | dict[str, list[str]] | dict[str, list[str | dict[str, list[str | dict[str, list[str]]]]]]]]
        | dict[str, list[str]]
    ]: ...
    @staticmethod
    def update_signature() -> list[str | dict[str, list[str | dict[str, list[str]]]] | dict[str, list[str]]]: ...
    @staticmethod
    def delete_signature() -> list[str]: ...



from _typeshed import Incomplete

def encode_client_secret_basic(client, method, uri, headers, body): ...
def encode_client_secret_post(client, method, uri, headers, body): ...
def encode_none(client, method, uri, headers, body): ...

class ClientAuth:
    DEFAULT_AUTH_METHODS: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    auth_method: Incomplete
    def __init__(self, client_id, client_secret, auth_method=None) -> None: ...
    def prepare(self, method, uri, headers, body): ...

class TokenAuth:
    DEFAULT_TOKEN_TYPE: str
    SIGN_METHODS: Incomplete
    token: Incomplete
    token_placement: Incomplete
    client: Incomplete
    hooks: Incomplete
    def __init__(self, token, token_placement: str = "header", client=None) -> None: ...
    def set_token(self, token) -> None: ...
    def prepare(self, uri, headers, body): ...
    def __del__(self) -> None: ...



from typing import Literal
from typing_extensions import TypeAlias

from docutils import parsers

_ParserName: TypeAlias = Literal["pycmark", "myst", "recommonmark"]

commonmark_parser_names: tuple[_ParserName, ...]
Parser: type[parsers.Parser]  # if Parser is None or parser_name is empty string, user cannot import current module
parser_name: _ParserName



from collections.abc import Iterator, Mapping
from typing import Any, NoReturn

from docker.models.containers import Container
from docker.models.images import Image
from requests import HTTPError, Response

class DockerException(Exception): ...

def create_api_error_from_http_exception(e: HTTPError) -> NoReturn: ...

class APIError(HTTPError, DockerException):
    response: Response | None
    explanation: str | None
    def __init__(self, message: str, response: Response | None = None, explanation: str | None = None) -> None: ...
    @property
    def status_code(self) -> int | None: ...
    def is_error(self) -> bool: ...
    def is_client_error(self) -> bool: ...
    def is_server_error(self) -> bool: ...

class NotFound(APIError): ...
class ImageNotFound(NotFound): ...
class InvalidVersion(DockerException): ...
class InvalidRepository(DockerException): ...
class InvalidConfigFile(DockerException): ...
class InvalidArgument(DockerException): ...
class DeprecatedMethod(DockerException): ...

class TLSParameterError(DockerException):
    msg: str
    def __init__(self, msg: str) -> None: ...

class NullResource(DockerException, ValueError): ...

class ContainerError(DockerException):
    container: Container
    exit_status: int
    command: str | list[str] | None
    image: str | Image
    stderr: str | None
    def __init__(
        self, container: Container, exit_status: int, command: str | list[str] | None, image: str | Image, stderr: str | None
    ) -> None: ...

class StreamParseError(RuntimeError):
    msg: str
    def __init__(self, reason: str) -> None: ...

class BuildError(DockerException):
    msg: str
    build_log: Iterator[dict[str, str]]
    def __init__(self, reason: str, build_log: Iterator[dict[str, str]]) -> None: ...

class ImageLoadError(DockerException): ...

def create_unexpected_kwargs_error(name, kwargs: Mapping[str, Any]) -> NoReturn: ...

class MissingContextParameter(DockerException):
    param: str
    def __init__(self, param: str) -> None: ...

class ContextAlreadyExists(DockerException):
    name: str
    def __init__(self, name: str) -> None: ...

class ContextException(DockerException):
    msg: str
    def __init__(self, msg: str) -> None: ...

class ContextNotFound(DockerException):
    name: str
    def __init__(self, name: str) -> None: ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Branding:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def get(self) -> dict[str, Incomplete]: ...
    async def get_async(self) -> dict[str, Incomplete]: ...
    def update(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_template_universal_login(self) -> dict[str, Incomplete]: ...
    async def get_template_universal_login_async(self) -> dict[str, Incomplete]: ...
    def delete_template_universal_login(self): ...
    async def delete_template_universal_login_async(self): ...
    def update_template_universal_login(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_template_universal_login_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def get_default_branding_theme(self) -> dict[str, Incomplete]: ...
    async def get_default_branding_theme_async(self) -> dict[str, Incomplete]: ...
    def get_branding_theme(self, theme_id: str) -> dict[str, Incomplete]: ...
    async def get_branding_theme_async(self, theme_id: str) -> dict[str, Incomplete]: ...
    def delete_branding_theme(self, theme_id: str): ...
    async def delete_branding_theme_async(self, theme_id: str): ...
    def update_branding_theme(self, theme_id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_branding_theme_async(self, theme_id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def create_branding_theme(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_branding_theme_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...



from typing import ClassVar, Final

from docutils.writers import html4css1

__docformat__: Final = "reStructuredText"

class Writer(html4css1.Writer):
    default_stylesheet: ClassVar[str]
    default_stylesheet_path: ClassVar[str]
    default_template_path: ClassVar[str]
    settings_default_overrides: ClassVar[dict[str, str]]
    relative_path_settings: ClassVar[tuple[str, ...]]
    config_section_dependencies: ClassVar[tuple[str, ...]]
    translator_class: type[HTMLTranslator]
    pepnum: str
    title: str
    def interpolation_dict(self) -> dict[str, str | int]: ...  # type: ignore[override]

class HTMLTranslator(html4css1.HTMLTranslator):
    def depart_field_list(self, node) -> None: ...



from typing import Final

from pika import adapters as adapters
from pika.adapters import (
    BaseConnection as BaseConnection,
    BlockingConnection as BlockingConnection,
    SelectConnection as SelectConnection,
)
from pika.adapters.utils.connection_workflow import AMQPConnectionWorkflow as AMQPConnectionWorkflow
from pika.connection import ConnectionParameters as ConnectionParameters, SSLOptions as SSLOptions, URLParameters as URLParameters
from pika.credentials import PlainCredentials as PlainCredentials
from pika.delivery_mode import DeliveryMode as DeliveryMode
from pika.spec import BasicProperties as BasicProperties

__version__: Final[str]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Add a dummy package name. Having no package like
core/lib/core/error_codes.proto, or having tensorflow like
tsl/protobuf/error_codes.proto, results in name collision errors in generated
code for some users that use JS through J2CL.
"""

import google.protobuf.descriptor
from tensorflow.tsl.protobuf.status_pb2 import StatusProto as StatusProto

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



from typing import Any

STATUS_INIT: str
STATUS_VIRTUAL: str
STATUS_MANDATORY_MISSING: str
STATUS_READ: str
STATUS_WRITABLE: str
STATUS_PENDING_CHANGES: str
STATUS_COMMITTED: str
STATUS_READY_FOR_DELETION: str
STATUS_READY_FOR_MOVING: str
STATUS_READY_FOR_RENAMING: str
STATUS_DELETED: str
STATUSES: Any
INITIAL_STATUSES: Any



from _typeshed import Incomplete

from antlr4.atn.ATNConfigSet import ATNConfigSet as ATNConfigSet
from antlr4.atn.ATNState import DecisionState as DecisionState, StarLoopEntryState as StarLoopEntryState
from antlr4.dfa.DFAState import DFAState as DFAState
from antlr4.error.Errors import IllegalStateException as IllegalStateException

class DFA:
    __slots__ = ("atnStartState", "decision", "_states", "s0", "precedenceDfa")
    atnStartState: Incomplete
    decision: Incomplete
    s0: Incomplete
    precedenceDfa: bool
    def __init__(self, atnStartState: DecisionState, decision: int = 0) -> None: ...
    def getPrecedenceStartState(self, precedence: int): ...
    def setPrecedenceStartState(self, precedence: int, startState: DFAState): ...
    def setPrecedenceDfa(self, precedenceDfa: bool): ...
    @property
    def states(self): ...
    def sortedStates(self): ...
    def toString(self, literalNames: list[str] | None = None, symbolicNames: list[str] | None = None): ...
    def toLexerString(self): ...



from _typeshed import FileDescriptorOrPath
from os import _Environ
from typing import AnyStr, Literal

from .exceptions import ExceptionPexpect
from .pty_spawn import spawn
from .spawnbase import _Logfile

__all__ = ["ExceptionPxssh", "pxssh"]

class ExceptionPxssh(ExceptionPexpect): ...

class pxssh(spawn[AnyStr]):
    name: str
    UNIQUE_PROMPT: str
    PROMPT: str
    PROMPT_SET_SH: str
    PROMPT_SET_CSH: str
    PROMPT_SET_ZSH: str
    SSH_OPTS: str
    force_password: bool
    debug_command_string: bool
    options: dict[str, str]
    def __init__(
        self,
        timeout: float | None = 30,
        maxread: int = 2000,
        searchwindowsize: int | None = None,
        logfile: _Logfile | None = None,
        cwd: FileDescriptorOrPath | None = None,
        env: _Environ[str] | None = None,
        ignore_sighup: bool = True,
        echo: bool = True,
        options: dict[str, str] = {},
        encoding: str | None = None,
        codec_errors: str = "strict",
        debug_command_string: bool = False,
        use_poll: bool = False,
    ) -> None: ...
    def levenshtein_distance(self, a, b): ...
    def try_read_prompt(self, timeout_multiplier): ...
    def sync_original_prompt(self, sync_multiplier: float = 1.0): ...
    def login(
        self,
        server,
        username: str | None = None,
        password: str = "",
        terminal_type: str = "ansi",
        original_prompt: str = "[#$]",
        login_timeout: float | None = 10,
        port: int | None = None,
        auto_prompt_reset: bool = True,
        ssh_key: FileDescriptorOrPath | Literal[True] | None = None,
        quiet: bool = True,
        sync_multiplier: int = 1,
        check_local_ip: bool = True,
        password_regex: str = "(?i)(?:password:)|(?:passphrase for key)",
        ssh_tunnels: dict[str, list[str | int]] = {},
        spawn_local_ssh: bool = True,
        sync_original_prompt: bool = True,
        ssh_config: FileDescriptorOrPath | None = None,
        cmd: str = "ssh",
    ): ...
    def logout(self) -> None: ...
    def prompt(self, timeout: float | None = -1): ...
    def set_unique_prompt(self): ...



from typing import Any, final
from weakref import ref

@final
class ValuedWeakRef(ref):
    __slots__ = ("value",)
    value: Any

@final
class IdentRegistry:
    def __init__(self) -> None: ...
    def get_ident(self, obj: object) -> int: ...
    def __len__(self) -> int: ...

__all__ = ["IdentRegistry"]



from nanoleafapi.nanoleaf import Nanoleaf

class NanoleafDigitalTwin:
    nanoleaf: Nanoleaf
    tile_dict: dict[str, dict[str, int]]
    def __init__(self, nl: Nanoleaf) -> None: ...
    def set_color(self, panel_id: int, rgb: tuple[int, int, int]) -> None: ...
    def set_all_colors(self, rgb: tuple[int, int, int]) -> None: ...
    def get_ids(self) -> list[int]: ...
    def get_color(self, panel_id: int) -> tuple[int, int, int]: ...
    def get_all_colors(self) -> dict[int, tuple[int, int, int]]: ...
    def sync(self) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["average_neighbor_degree"]

@_dispatchable
def average_neighbor_degree(
    G: Graph[_Node],
    source: str | None = "out",
    target: str | None = "out",
    nodes: Iterable[Incomplete] | None = None,
    weight: str | None = None,
) -> dict[Incomplete, Incomplete]: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import Widget
from reportlab.lib.attrmap import *
from reportlab.lib.validators import *

__version__: Final[str]

class TableWidget(Widget):
    x: Incomplete
    y: Incomplete
    width: int
    height: int
    borderStrokeColor: Incomplete
    fillColor: Incomplete
    borderStrokeWidth: float
    horizontalDividerStrokeColor: Incomplete
    verticalDividerStrokeColor: Incomplete
    horizontalDividerStrokeWidth: float
    verticalDividerStrokeWidth: float
    dividerDashArray: Incomplete
    data: Incomplete
    boxAnchor: str
    fontSize: int
    fontColor: Incomplete
    alignment: str
    textAnchor: str
    def __init__(self, x: int = 10, y: int = 10, **kw) -> None: ...
    def demo(self): ...
    def draw(self): ...
    def preProcessData(self, data): ...



from _typeshed import StrPath, Unused

from setuptools.dist import Distribution

from .._distutils.command import install_lib as orig

class install_lib(orig.install_lib):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    def run(self) -> None: ...
    def get_exclusions(self): ...
    def copy_tree(
        self,
        infile: StrPath,
        outfile: str,
        preserve_mode: bool = True,
        preserve_times: bool = True,
        preserve_symlinks: bool = False,
        level: Unused = 1,
    ): ...
    def get_outputs(self): ...



from _typeshed import StrPath
from _typeshed.wsgi import WSGIApplication
from collections.abc import Iterator
from typing import IO, Any

from webob.dec import wsgify
from webob.request import Request
from webob.response import Response

__all__ = ["FileApp", "DirectoryApp"]

BLOCK_SIZE: int

class FileApp:
    filename: StrPath
    kw: dict[str, Any]
    def __init__(self, filename: StrPath, **kw: Any) -> None: ...
    @wsgify
    def __call__(self, req: Request) -> WSGIApplication: ...

class FileIter:
    file: IO[bytes]
    def __init__(self, file: IO[bytes]) -> None: ...
    def app_iter_range(
        self, seek: int | None = None, limit: int | None = None, block_size: int | None = None
    ) -> Iterator[bytes]: ...
    __iter__ = app_iter_range

class DirectoryApp:
    path: StrPath
    index_page: str
    hide_index_with_redirect: bool
    fileapp_kw: dict[str, Any]
    def __init__(
        self, path: StrPath, index_page: str = "index.html", hide_index_with_redirect: bool = False, **kw: Any
    ) -> None: ...
    def make_fileapp(self, path: StrPath) -> FileApp: ...
    @wsgify
    def __call__(self, req: Request) -> Response | FileApp: ...
    def index(self, req: Request, path: StrPath) -> Response | FileApp: ...



from _typeshed import Incomplete

from braintree.add_on import AddOn

class AddOnGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def all(self) -> list[AddOn]: ...



from typing_extensions import TypeAlias

class BasePriorityQueue:
    def __init__(self, **kw) -> None: ...
    def add(self, task, priority: int | None = None) -> None: ...
    def remove(self, task) -> None: ...
    def peek(self, default=...): ...
    def pop(self, default=...): ...
    def __len__(self) -> int: ...

class HeapPriorityQueue(BasePriorityQueue): ...
class SortedPriorityQueue(BasePriorityQueue): ...

PriorityQueue: TypeAlias = SortedPriorityQueue

__all__ = ["PriorityQueue", "BasePriorityQueue", "HeapPriorityQueue", "SortedPriorityQueue"]



from ..filters import (
    AllValuesFilter,
    AllValuesMultipleFilter,
    BaseCSVFilter,
    BaseInFilter,
    BaseRangeFilter,
    BooleanFilter as _BaseBooleanFilter,
    CharFilter,
    ChoiceFilter,
    DateFilter,
    DateFromToRangeFilter,
    DateRangeFilter,
    DateTimeFilter,
    DateTimeFromToRangeFilter,
    DurationFilter,
    Filter,
    IsoDateTimeFilter,
    IsoDateTimeFromToRangeFilter,
    LookupChoiceFilter,
    ModelChoiceFilter,
    ModelMultipleChoiceFilter,
    MultipleChoiceFilter,
    NumberFilter,
    NumericRangeFilter,
    OrderingFilter,
    RangeFilter,
    TimeFilter,
    TimeRangeFilter,
    TypedChoiceFilter,
    TypedMultipleChoiceFilter,
    UUIDFilter,
)

__all__ = [
    "AllValuesFilter",
    "AllValuesMultipleFilter",
    "BaseCSVFilter",
    "BaseInFilter",
    "BaseRangeFilter",
    "BooleanFilter",
    "CharFilter",
    "ChoiceFilter",
    "DateFilter",
    "DateFromToRangeFilter",
    "DateRangeFilter",
    "DateTimeFilter",
    "DateTimeFromToRangeFilter",
    "DurationFilter",
    "Filter",
    "IsoDateTimeFilter",
    "IsoDateTimeFromToRangeFilter",
    "LookupChoiceFilter",
    "ModelChoiceFilter",
    "ModelMultipleChoiceFilter",
    "MultipleChoiceFilter",
    "NumberFilter",
    "NumericRangeFilter",
    "OrderingFilter",
    "RangeFilter",
    "TimeFilter",
    "TimeRangeFilter",
    "TypedChoiceFilter",
    "TypedMultipleChoiceFilter",
    "UUIDFilter",
]

# REST framework specific BooleanFilter that uses BooleanWidget by default
class BooleanFilter(_BaseBooleanFilter): ...



from collections.abc import ItemsView, Mapping, MutableMapping
from typing import Any, TypeVar

_KT = TypeVar("_KT")
_VT_co = TypeVar("_VT_co", covariant=True)
_MM = TypeVar("_MM", bound=MutableMapping[Any, Any])

def url_join(base: str, *args: str) -> str: ...
def copy_kwargs(dictionary: _MM) -> _MM: ...
def iterator(d: Mapping[_KT, _VT_co]) -> ItemsView[_KT, _VT_co]: ...



from braintree.ach_mandate import AchMandate
from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult
from braintree.us_bank_account_verification import UsBankAccountVerification

class UsBankAccount(Resource):
    @staticmethod
    def find(token: str) -> UsBankAccount | None: ...
    @staticmethod
    def sale(token: str, transactionRequest) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def signature() -> list[str]: ...
    ach_mandate: AchMandate | None
    verifications: list[UsBankAccountVerification]
    def __init__(self, gateway, attributes) -> None: ...



from typing import overload

import numpy as np
from numpy.typing import NDArray

from ._typing import ArrayLike, ArrayLikeSeq, OptGeoArrayLike, OptGeoArrayLikeSeq
from .lib import Geometry

__all__ = [
    "area",
    "bounds",
    "distance",
    "frechet_distance",
    "hausdorff_distance",
    "length",
    "minimum_bounding_radius",
    "minimum_clearance",
    "total_bounds",
]

@overload
def area(geometry: Geometry | None, **kwargs) -> np.float64: ...
@overload
def area(geometry: OptGeoArrayLikeSeq, **kwargs) -> NDArray[np.float64]: ...
@overload
def distance(a: Geometry | None, b: Geometry | None, **kwargs) -> np.float64: ...
@overload
def distance(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, **kwargs) -> NDArray[np.float64]: ...
@overload
def distance(a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, **kwargs) -> NDArray[np.float64]: ...
def bounds(geometry: OptGeoArrayLike, **kwargs) -> NDArray[np.float64]: ...
def total_bounds(geometry: OptGeoArrayLike, **kwargs) -> NDArray[np.float64]: ...
@overload
def length(geometry: Geometry | None, **kwargs) -> np.float64: ...
@overload
def length(geometry: OptGeoArrayLikeSeq, **kwargs) -> NDArray[np.float64]: ...
@overload
def hausdorff_distance(a: Geometry | None, b: Geometry | None, densify: float | None = None, **kwargs) -> np.float64: ...
@overload
def hausdorff_distance(a: OptGeoArrayLike, b: OptGeoArrayLike, densify: ArrayLikeSeq[float], **kwargs) -> NDArray[np.float64]: ...
@overload
def hausdorff_distance(
    a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, densify: ArrayLike[float] | None = None, **kwargs
) -> NDArray[np.float64]: ...
@overload
def hausdorff_distance(
    a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, densify: ArrayLike[float] | None = None, **kwargs
) -> NDArray[np.float64]: ...
@overload
def frechet_distance(a: Geometry | None, b: Geometry | None, densify: float | None = None, **kwargs) -> np.float64: ...
@overload
def frechet_distance(a: OptGeoArrayLike, b: OptGeoArrayLike, densify: ArrayLikeSeq[float], **kwargs) -> NDArray[np.float64]: ...
@overload
def frechet_distance(
    a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, densify: ArrayLike[float] | None = None, **kwargs
) -> NDArray[np.float64]: ...
@overload
def frechet_distance(
    a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, densify: ArrayLike[float] | None = None, **kwargs
) -> NDArray[np.float64]: ...
@overload
def minimum_clearance(geometry: Geometry | None, **kwargs) -> np.float64: ...
@overload
def minimum_clearance(geometry: OptGeoArrayLikeSeq, **kwargs) -> NDArray[np.float64]: ...
@overload
def minimum_bounding_radius(geometry: Geometry | None, **kwargs) -> np.float64: ...
@overload
def minimum_bounding_radius(geometry: OptGeoArrayLikeSeq, **kwargs) -> NDArray[np.float64]: ...



from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from typing import Final, NamedTuple
from uuid import UUID

from .enums import Align, TextDirection, WrapMode

SOFT_HYPHEN: Final[str]
HYPHEN: Final[str]
SPACE: Final[str]
BREAKING_SPACE_SYMBOLS: Final[list[str]]
BREAKING_SPACE_SYMBOLS_STR: Final[str]
NBSP: Final[str]
NEWLINE: Final[str]
FORM_FEED: Final[str]

class Fragment:
    characters: list[str]
    graphics_state: dict[str, Incomplete]
    k: float
    url: str | None
    def __init__(
        self, characters: list[str] | str, graphics_state: dict[str, Incomplete], k: float, link: str | int | None = None
    ) -> None: ...
    @property
    def font(self): ...
    @font.setter
    def font(self, v) -> None: ...
    @property
    def is_ttf_font(self): ...
    @property
    def font_style(self): ...
    @property
    def font_family(self): ...
    @property
    def font_size_pt(self): ...
    @property
    def font_size(self): ...
    @property
    def font_stretching(self): ...
    @property
    def char_spacing(self): ...
    @property
    def text_mode(self): ...
    @property
    def underline(self) -> bool: ...
    @property
    def strikethrough(self) -> bool: ...
    @property
    def draw_color(self): ...
    @property
    def fill_color(self): ...
    @property
    def text_color(self): ...
    @property
    def line_width(self): ...
    @property
    def char_vpos(self): ...
    @property
    def lift(self): ...
    @property
    def string(self) -> str: ...
    @property
    def width(self) -> float: ...
    @property
    def text_shaping_parameters(self): ...
    @property
    def paragraph_direction(self) -> TextDirection: ...
    @property
    def fragment_direction(self) -> TextDirection: ...
    def trim(self, index: int) -> None: ...
    def __eq__(self, other: Fragment) -> bool: ...  # type: ignore[override]
    def get_width(self, start: int = 0, end: int | None = None, chars: str | None = None, initial_cs: bool = True) -> float: ...
    def has_same_style(self, other: Fragment) -> bool: ...
    def get_character_width(self, character: str, print_sh: bool = False, initial_cs: bool = True): ...
    def render_pdf_text(self, frag_ws, current_ws, word_spacing, adjust_x, adjust_y, h): ...
    def render_pdf_text_ttf(self, frag_ws, word_spacing): ...
    def render_with_text_shaping(self, pos_x: float, pos_y: float, h: float, word_spacing: float) -> str: ...
    def render_pdf_text_core(self, frag_ws, current_ws): ...

class TotalPagesSubstitutionFragment(Fragment):
    uuid: UUID
    def get_placeholder_string(self) -> str: ...
    def render_text_substitution(self, replacement_text: str) -> str: ...

class TextLine(NamedTuple):
    fragments: tuple[Fragment, ...]
    text_width: float
    number_of_spaces: int
    align: Align
    height: float
    max_width: float
    trailing_nl: bool = False
    trailing_form_feed: bool = False
    indent: float = 0
    def get_ordered_fragments(self) -> tuple[Fragment, ...]: ...

class SpaceHint(NamedTuple):
    original_fragment_index: int
    original_character_index: int
    current_line_fragment_index: int
    current_line_character_index: int
    line_width: float
    number_of_spaces: int

class HyphenHint(NamedTuple):
    original_fragment_index: int
    original_character_index: int
    current_line_fragment_index: int
    current_line_character_index: int
    line_width: float
    number_of_spaces: int
    curchar: str
    curchar_width: float
    graphics_state: dict[str, Incomplete]
    k: float

class CurrentLine:
    max_width: float
    print_sh: bool
    indent: float
    fragments: list[Fragment]
    height: int
    number_of_spaces: int
    space_break_hint: Incomplete
    hyphen_break_hint: Incomplete
    def __init__(self, max_width: float, print_sh: bool = False, indent: float = 0) -> None: ...
    @property
    def width(self) -> float: ...
    def add_character(
        self,
        character: str,
        character_width: float,
        original_fragment: Fragment,
        original_fragment_index: int,
        original_character_index: int,
        height: float,
        url: str | None = None,
    ) -> None: ...
    def trim_trailing_spaces(self) -> None: ...
    def manual_break(self, align: Align, trailing_nl: bool = False, trailing_form_feed: bool = False) -> TextLine: ...
    def automatic_break_possible(self) -> bool: ...
    def automatic_break(self, align: Align) -> tuple[Incomplete, Incomplete, TextLine]: ...

class MultiLineBreak:
    fragments: Sequence[Fragment]
    get_width: float
    margins: Sequence[float]
    align: Align
    print_sh: bool
    wrapmode: WrapMode
    line_height: float
    skip_leading_spaces: bool
    fragment_index: int
    character_index: int
    idx_last_forced_break: int | None
    first_line_indent: float
    def __init__(
        self,
        fragments: Sequence[Fragment],
        max_width: float | Callable[[float], float],
        margins: Sequence[float],
        align: Align = ...,
        print_sh: bool = False,
        wrapmode: WrapMode = ...,
        line_height: float = 1.0,
        skip_leading_spaces: bool = False,
        first_line_indent: float = 0,
    ) -> None: ...
    def get_line(self) -> TextLine: ...



from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Auth(SystemBackendMixin):
    def list_auth_methods(self): ...
    def enable_auth_method(
        self, method_type, description=None, config=None, plugin_name=None, local: bool = False, path=None, **kwargs
    ): ...
    def disable_auth_method(self, path): ...
    def read_auth_method_tuning(self, path): ...
    def tune_auth_method(
        self,
        path,
        default_lease_ttl=None,
        max_lease_ttl=None,
        description=None,
        audit_non_hmac_request_keys=None,
        audit_non_hmac_response_keys=None,
        listing_visibility=None,
        passthrough_request_headers=None,
        **kwargs,
    ): ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["line_graph", "inverse_line_graph"]

@_dispatchable
def line_graph(G: Graph[_Node], create_using=None): ...
@_dispatchable
def inverse_line_graph(G: Graph[_Node]): ...



from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNDeserializer import ATNDeserializer as ATNDeserializer
from antlr4.atn.LexerATNSimulator import LexerATNSimulator as LexerATNSimulator
from antlr4.atn.ParserATNSimulator import ParserATNSimulator as ParserATNSimulator
from antlr4.atn.PredictionMode import PredictionMode as PredictionMode
from antlr4.BufferedTokenStream import TokenStream as TokenStream
from antlr4.CommonTokenStream import CommonTokenStream as CommonTokenStream
from antlr4.dfa.DFA import DFA as DFA
from antlr4.error.DiagnosticErrorListener import DiagnosticErrorListener as DiagnosticErrorListener
from antlr4.error.Errors import (
    IllegalStateException as IllegalStateException,
    NoViableAltException as NoViableAltException,
    RecognitionException as RecognitionException,
)
from antlr4.error.ErrorStrategy import BailErrorStrategy as BailErrorStrategy
from antlr4.FileStream import FileStream as FileStream
from antlr4.InputStream import InputStream as InputStream
from antlr4.Lexer import Lexer as Lexer
from antlr4.Parser import Parser as Parser
from antlr4.ParserRuleContext import ParserRuleContext as ParserRuleContext, RuleContext as RuleContext
from antlr4.PredictionContext import PredictionContextCache as PredictionContextCache
from antlr4.StdinStream import StdinStream as StdinStream
from antlr4.Token import Token as Token
from antlr4.tree.Tree import (
    ErrorNode as ErrorNode,
    ParseTreeListener as ParseTreeListener,
    ParseTreeVisitor as ParseTreeVisitor,
    ParseTreeWalker as ParseTreeWalker,
    RuleNode as RuleNode,
    TerminalNode as TerminalNode,
)
from antlr4.Utils import str_list as str_list



from _typeshed import Incomplete

from pony.orm import *
from pony.orm.core import Database, Entity

database: Database

class User(Entity):
    __slots__ = ()
    user_id: PrimaryKey
    owned_pokemons: Incomplete
    is_admin: Incomplete
    is_banned: Incomplete
    @staticmethod
    def get_or_create(user_id: int) -> User: ...
    @staticmethod
    def get_by_id(user_id: int) -> User: ...
    def catch_pokemon(self, pokemon: Pokemon): ...
    def remove_pokemon(self, pokemon: Pokemon): ...
    favorite_color: Incomplete
    def set_favorite_color(self, color: tuple[Incomplete, ...]): ...

class Pokemon(Entity):
    __slots__ = ()
    name: Incomplete
    pokemon_id: Incomplete
    sprite: Incomplete
    is_shiny: Incomplete
    owner: Incomplete
    spawned_chat_id: Incomplete
    spawned_message_id: Incomplete
    @property
    def captured(self) -> bool: ...
    def caught_by(self, user: User): ...

class Chat(Entity):
    __slots__ = ()
    chat_id: Incomplete
    active: Incomplete
    def activate(self) -> None: ...
    def deactivate(self) -> None: ...
    @staticmethod
    def get_or_create(chat_id: int) -> Chat: ...
    @staticmethod
    def get_by_id(chat_id: int) -> Chat: ...

def spawn_pokemon(
    chat_id: int, message_id: int, pokemon_json: dict[Incomplete, Incomplete], is_shiny: bool = False
) -> Pokemon: ...
def get_spawned_pokemon(chat_id: int, message_id: int) -> Pokemon | None: ...
def setup() -> None: ...



from _typeshed import Incomplete
from typing_extensions import Self

class PhoneInput:
    def __init__(
        self, country_phone_code: str | None = None, phone_number: str | None = None, extension_number: str | None = None
    ) -> None: ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...
    @staticmethod
    def builder() -> Builder: ...

    class Builder:
        def __init__(self) -> None: ...
        def country_phone_code(self, country_phone_code: str) -> Self: ...
        def phone_number(self, phone_number: str) -> Self: ...
        def extension_number(self, extension_number: str) -> Self: ...
        def build(self) -> PhoneInput: ...



from antlr4.InputStream import InputStream as InputStream

class FileStream(InputStream):
    __slots__ = "fileName"
    fileName: str
    def __init__(self, fileName: str, encoding: str = "ascii", errors: str = "strict") -> None: ...
    def readDataFrom(self, fileName: str, encoding: str, errors: str = "strict") -> str: ...



from _typeshed import ReadableBuffer
from logging import Logger
from typing import Any, Final

log: Logger

BASE64_CHARS: Final[str]
AB64_CHARS: Final[str]
HASH64_CHARS: Final[str]
BCRYPT_CHARS: Final[str]
PADDED_BASE64_CHARS: Final[str]
HEX_CHARS: Final[str]
UPPER_HEX_CHARS: Final[str]
LOWER_HEX_CHARS: Final[str]
ALL_BYTE_VALUES: Final[bytes]

def compile_byte_translation(mapping: dict[str | bytes | int, str | bytes], source: bytes | None = None) -> bytes: ...
def b64s_encode(data: ReadableBuffer) -> bytes: ...
def b64s_decode(data: str | ReadableBuffer) -> bytes: ...
def ab64_encode(data: ReadableBuffer) -> bytes: ...
def ab64_decode(data: str | ReadableBuffer) -> bytes: ...
def b32encode(source: ReadableBuffer) -> str: ...
def b32decode(source: str | bytes) -> bytes: ...

class Base64Engine:
    bytemap: Any
    big: Any
    def __init__(self, charmap, big: bool = False) -> None: ...
    @property
    def charmap(self): ...
    def encode_bytes(self, source): ...
    def decode_bytes(self, source): ...
    def check_repair_unused(self, source): ...
    def repair_unused(self, source): ...
    def encode_transposed_bytes(self, source, offsets): ...
    def decode_transposed_bytes(self, source, offsets): ...
    def decode_int6(self, source): ...
    def decode_int12(self, source): ...
    def decode_int24(self, source): ...
    def decode_int30(self, source): ...
    def decode_int64(self, source): ...
    def encode_int6(self, value): ...
    def encode_int12(self, value): ...
    def encode_int24(self, value): ...
    def encode_int30(self, value): ...
    def encode_int64(self, value): ...

class LazyBase64Engine(Base64Engine):
    def __init__(self, *args, **kwds) -> None: ...
    def __getattribute__(self, attr: str): ...

h64: Base64Engine
h64big: Base64Engine
bcrypt64: Base64Engine

__all__ = [
    # constants
    "BASE64_CHARS",
    "PADDED_BASE64_CHARS",
    "AB64_CHARS",
    "HASH64_CHARS",
    "BCRYPT_CHARS",
    "HEX_CHARS",
    "LOWER_HEX_CHARS",
    "UPPER_HEX_CHARS",
    "ALL_BYTE_VALUES",
    # misc
    "compile_byte_translation",
    # base64
    "ab64_encode",
    "ab64_decode",
    "b64s_encode",
    "b64s_decode",
    # base32
    "b32encode",
    "b32decode",
    # custom encodings
    "Base64Engine",
    "LazyBase64Engine",
    "h64",
    "h64big",
    "bcrypt64",
]



from _typeshed import Incomplete
from collections.abc import Callable

from oauthlib.oauth2.rfc6749.clients.base import Client, _TokenPlacement

class DeviceClient(Client):
    grant_type: str
    client_secret: str | None
    def __init__(
        self,
        client_id: str,
        *,
        client_secret: str | None = None,
        default_token_placement: _TokenPlacement = "auth_header",
        token_type: str = "Bearer",
        access_token: str | None = None,
        refresh_token: str | None = None,
        mac_key: str | bytes | bytearray | None = None,
        mac_algorithm: str | None = None,
        token: dict[str, Incomplete] | None = None,
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        state: str | None = None,
        redirect_url: str | None = None,
        state_generator: Callable[[], str] = ...,
        code_verifier: str | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = None,
        **kwargs,
    ) -> None: ...
    def prepare_request_uri(
        self, uri: str, scope: str | set[object] | tuple[object] | list[object] | None = None, **kwargs
    ) -> str: ...
    def prepare_request_body(
        self,
        device_code: str,
        body: str = "",
        scope: str | set[object] | tuple[object] | list[object] | None = None,
        include_client_id: bool = False,
        **kwargs,
    ) -> str: ...



from .authorization_server import AuthorizationServer as AuthorizationServer
from .client_auth import ClientAuth as ClientAuth
from .models import (
    ClientMixin as ClientMixin,
    TemporaryCredential as TemporaryCredential,
    TemporaryCredentialMixin as TemporaryCredentialMixin,
    TokenCredentialMixin as TokenCredentialMixin,
)
from .resource_protector import ResourceProtector as ResourceProtector
from .signature import (
    SIGNATURE_HMAC_SHA1 as SIGNATURE_HMAC_SHA1,
    SIGNATURE_PLAINTEXT as SIGNATURE_PLAINTEXT,
    SIGNATURE_RSA_SHA1 as SIGNATURE_RSA_SHA1,
    SIGNATURE_TYPE_BODY as SIGNATURE_TYPE_BODY,
    SIGNATURE_TYPE_HEADER as SIGNATURE_TYPE_HEADER,
    SIGNATURE_TYPE_QUERY as SIGNATURE_TYPE_QUERY,
)
from .wrapper import OAuth1Request as OAuth1Request

__all__ = [
    "OAuth1Request",
    "ClientAuth",
    "SIGNATURE_HMAC_SHA1",
    "SIGNATURE_RSA_SHA1",
    "SIGNATURE_PLAINTEXT",
    "SIGNATURE_TYPE_HEADER",
    "SIGNATURE_TYPE_QUERY",
    "SIGNATURE_TYPE_BODY",
    "ClientMixin",
    "TemporaryCredentialMixin",
    "TokenCredentialMixin",
    "TemporaryCredential",
    "AuthorizationServer",
    "ResourceProtector",
]



import codecs
from collections.abc import Iterable, Iterator
from typing import Final

VERSION: Final[str]
PYTHON_NAMES: Final[dict[str, str]]
CACHE: dict[str, Encoding]

def ascii_lower(string: str) -> str: ...
def lookup(label: str) -> Encoding | None: ...

class Encoding:
    name: str
    codec_info: codecs.CodecInfo
    def __init__(self, name: str, codec_info: codecs.CodecInfo) -> None: ...

UTF8: Final[Encoding]

def decode(input: bytes | bytearray, fallback_encoding: str | Encoding, errors: str = "replace") -> tuple[str, Encoding]: ...
def encode(input: str, encoding: str | Encoding = ..., errors: str = "strict") -> bytes: ...
def iter_decode(
    input: Iterable[bytes | bytearray], fallback_encoding: str | Encoding, errors: str = "replace"
) -> tuple[Iterator[Encoding | str], Encoding]: ...
def iter_encode(input: Iterable[str], encoding: str | Encoding = ..., errors: str = "strict") -> Iterator[bytes]: ...

class IncrementalDecoder:
    encoding: Encoding | None
    def __init__(self, fallback_encoding: str | Encoding, errors: str = "replace") -> None: ...
    def decode(self, input: bytes | bytearray, final: bool = False) -> str: ...

class IncrementalEncoder:
    encode: bytes
    def __init__(self, encoding: str | Encoding = ..., errors: str = "strict") -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Protobuf containing the metadata for each Keras object saved in a SavedModel."""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.python.keras.protobuf.versions_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SavedMetadata(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODES_FIELD_NUMBER: builtins.int
    @property
    def nodes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SavedObject]:
        """Nodes represent trackable objects in the SavedModel. The data for every
        Keras object is stored.
        """

    def __init__(self, *, nodes: collections.abc.Iterable[global___SavedObject] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["nodes", b"nodes"]) -> None: ...

global___SavedMetadata = SavedMetadata

@typing.final
class SavedObject(google.protobuf.message.Message):
    """Metadata of an individual Keras object."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NODE_ID_FIELD_NUMBER: builtins.int
    NODE_PATH_FIELD_NUMBER: builtins.int
    IDENTIFIER_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    node_id: builtins.int
    """Index of the node in the SavedModel SavedObjectGraph."""
    node_path: builtins.str
    """String path from root (e.g. "root.child_layer")"""
    identifier: builtins.str
    """Identifier to determine loading function.
    Must be one of:
      _tf_keras_input_layer, _tf_keras_layer, _tf_keras_metric,
      _tf_keras_model, _tf_keras_network, _tf_keras_rnn_layer,
      _tf_keras_sequential
    """
    metadata: builtins.str
    """Metadata containing a JSON-serialized object with the non-TensorFlow
    attributes for this Keras object.
    """
    @property
    def version(self) -> tensorflow.python.keras.protobuf.versions_pb2.VersionDef:
        """Version defined by the code serializing this Keras object."""

    def __init__(
        self,
        *,
        node_id: builtins.int | None = ...,
        node_path: builtins.str | None = ...,
        identifier: builtins.str | None = ...,
        metadata: builtins.str | None = ...,
        version: tensorflow.python.keras.protobuf.versions_pb2.VersionDef | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["version", b"version"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "identifier",
            b"identifier",
            "metadata",
            b"metadata",
            "node_id",
            b"node_id",
            "node_path",
            b"node_path",
            "version",
            b"version",
        ],
    ) -> None: ...

global___SavedObject = SavedObject



from collections.abc import Callable, Iterable
from typing import Any, Final, TypeVar

_T = TypeVar("_T")

UNICODE_ASCII_CHARACTER_SET: Final[str]

def filter_params(
    target: Callable[[dict[str, Any] | Iterable[tuple[str, Any]], _T], object],
) -> Callable[[list[str], _T], object]: ...
def filter_oauth_params(
    params: dict[str, Any] | Iterable[tuple[str, Any]],
) -> list[str]: ...  # we don't care about second (Any) part
def escape(u: str) -> str: ...
def unescape(u: str) -> str: ...
def parse_keqv_list(l: list[str]) -> dict[str, str]: ...
def parse_http_list(u: str) -> list[str]: ...
def parse_authorization_header(authorization_header: str) -> list[tuple[str, str]]: ...



from typing import Final, Literal

from Xlib.display import Display
from Xlib.error import XError
from Xlib.protocol import request, rq
from Xlib.xobject import resource

extname: Final = "DAMAGE"
DamageNotifyCode: Final = 0
BadDamageCode: Final = 0
DamageReportRawRectangles: Final = 0
DamageReportDeltaRectangles: Final = 1
DamageReportBoundingBox: Final = 2
DamageReportNonEmpty: Final = 3
DamageReportLevel: Final[tuple[Literal[0], Literal[1], Literal[2], Literal[3]]]
DAMAGE = rq.Card32

class BadDamageError(XError): ...
class QueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource) -> QueryVersion: ...

class DamageCreate(rq.Request): ...

def damage_create(self: Display | resource.Resource, level: int) -> int: ...

class DamageDestroy(rq.Request): ...

def damage_destroy(self: Display | resource.Resource, damage: int) -> None: ...

class DamageSubtract(rq.Request): ...

def damage_subtract(self: Display | resource.Resource, damage: int, repair: int = 0, parts: int = 0) -> None: ...

class DamageAdd(rq.Request): ...

def damage_add(self: Display | resource.Resource, repair: int, parts: int) -> None: ...

class DamageNotify(rq.Event): ...

def init(disp: Display, info: request.QueryExtension) -> None: ...



from typing import Literal

from ._canonical_names import canonical_names as canonical_names, normalize_name as normalize_name

KEY_DOWN: Literal["down"]
KEY_UP: Literal["up"]

class KeyboardEvent:
    event_type: Literal["down", "up"] | None
    scan_code: int
    name: str | None
    time: float | None
    device: str | None
    modifiers: tuple[str, ...] | None
    is_keypad: bool | None

    def __init__(
        self,
        event_type: Literal["down", "up"] | None,
        scan_code: int,
        name: str | None = None,
        time: float | None = None,
        device: str | None = None,
        modifiers: tuple[str, ...] | None = None,
        is_keypad: bool | None = None,
    ) -> None: ...
    def to_json(self, ensure_ascii: bool = False) -> str: ...
    def __eq__(self, other: object) -> bool: ...



from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["local_node_connectivity", "node_connectivity", "all_pairs_node_connectivity"]

@_dispatchable
def local_node_connectivity(G: Graph[_Node], source: _Node, target: _Node, cutoff: float | None = None) -> float: ...
@_dispatchable
def node_connectivity(G: Graph[_Node], s: _Node | None = None, t: _Node | None = None) -> float: ...
@_dispatchable
def all_pairs_node_connectivity(
    G: Graph[_Node], nbunch: Iterable[_Node] | None = None, cutoff: float | None = None
) -> dict[_Node, dict[_Node, float]]: ...



from _typeshed import SupportsRead, SupportsWrite
from os import PathLike
from typing import Any

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from pydot import Dot  # type: ignore[import-not-found]  # pyright: ignore[reportMissingImports]

__all__ = ["write_dot", "read_dot", "graphviz_layout", "pydot_layout", "to_pydot", "from_pydot"]

def write_dot(G: Graph[_Node], path: str | PathLike[Any] | SupportsWrite[str]) -> None: ...
@_dispatchable
def read_dot(path: str | PathLike[Any] | SupportsRead[str]) -> Graph[str]: ...
@_dispatchable
def from_pydot(P: Dot) -> Graph[str]: ...
def to_pydot(N: Graph[_Node]) -> Dot: ...
def graphviz_layout(G: Graph[_Node], prog: str = "neato", root: _Node | None = None) -> dict[_Node, tuple[float, float]]: ...
def pydot_layout(G: Graph[_Node], prog: str = "neato", root: _Node | None = None) -> dict[_Node, tuple[float, float]]: ...



__all__ = [
    "powerlaw_sequence",
    "zipf_rv",
    "cumulative_distribution",
    "discrete_sequence",
    "random_weighted_sample",
    "weighted_choice",
]

def powerlaw_sequence(n, exponent: float = 2.0, seed=None): ...
def zipf_rv(alpha, xmin: int = 1, seed=None) -> int: ...
def cumulative_distribution(distribution): ...
def discrete_sequence(n, distribution=None, cdistribution=None, seed=None): ...
def random_weighted_sample(mapping, k, seed=None): ...
def weighted_choice(mapping, seed=None): ...



from typing import Final

HTTP_CONTINUE: Final = 100
HTTP_SWITCHING_PROTOCOLS: Final = 101
HTTP_PROCESSING: Final = 102
HTTP_OK: Final = 200
HTTP_CREATED: Final = 201
HTTP_ACCEPTED: Final = 202
HTTP_NON_AUTHORITATIVE: Final = 203
HTTP_NO_CONTENT: Final = 204
HTTP_RESET_CONTENT: Final = 205
HTTP_PARTIAL_CONTENT: Final = 206
HTTP_MULTI_STATUS: Final = 207
HTTP_MULTIPLE_CHOICES: Final = 300
HTTP_MOVED_PERMANENTLY: Final = 301
HTTP_MOVED_TEMPORARILY: Final = 302
HTTP_SEE_OTHER: Final = 303
HTTP_NOT_MODIFIED: Final = 304
HTTP_USE_PROXY: Final = 305
HTTP_TEMPORARY_REDIRECT: Final = 307
HTTP_BAD_REQUEST: Final = 400
HTTP_UNAUTHORIZED: Final = 401
HTTP_PAYMENT_REQUIRED: Final = 402
HTTP_FORBIDDEN: Final = 403
HTTP_NOT_FOUND: Final = 404
HTTP_METHOD_NOT_ALLOWED: Final = 405
HTTP_NOT_ACCEPTABLE: Final = 406
HTTP_PROXY_AUTHENTICATION_REQUIRED: Final = 407
HTTP_REQUEST_TIME_OUT: Final = 408
HTTP_CONFLICT: Final = 409
HTTP_GONE: Final = 410
HTTP_LENGTH_REQUIRED: Final = 411
HTTP_PRECONDITION_FAILED: Final = 412
HTTP_REQUEST_ENTITY_TOO_LARGE: Final = 413
HTTP_REQUEST_URI_TOO_LARGE: Final = 414
HTTP_UNSUPPORTED_MEDIA_TYPE: Final = 415
HTTP_RANGE_NOT_SATISFIABLE: Final = 416
HTTP_EXPECTATION_FAILED: Final = 417
HTTP_UNPROCESSABLE_ENTITY: Final = 422
HTTP_INTERNAL_SERVER_ERROR: Final = 500
HTTP_NOT_IMPLEMENTED: Final = 501
HTTP_BAD_GATEWAY: Final = 502
HTTP_SERVICE_UNAVAILABLE: Final = 503
HTTP_GATEWAY_TIME_OUT: Final = 504
HTTP_VERSION_NOT_SUPPORTED: Final = 505
HTTP_VARIANT_ALSO_VARIES: Final = 506
HSE_STATUS_SUCCESS: Final = 1
HSE_STATUS_SUCCESS_AND_KEEP_CONN: Final = 2
HSE_STATUS_PENDING: Final = 3
HSE_STATUS_ERROR: Final = 4
SF_NOTIFY_SECURE_PORT: Final = 0x00000001
SF_NOTIFY_NONSECURE_PORT: Final = 0x00000002
SF_NOTIFY_READ_RAW_DATA: Final = 0x00008000
SF_NOTIFY_PREPROC_HEADERS: Final = 0x00004000
SF_NOTIFY_AUTHENTICATION: Final = 0x00002000
SF_NOTIFY_URL_MAP: Final = 0x00001000
SF_NOTIFY_ACCESS_DENIED: Final = 0x00000800
SF_NOTIFY_SEND_RESPONSE: Final = 0x00000040
SF_NOTIFY_SEND_RAW_DATA: Final = 0x00000400
SF_NOTIFY_LOG: Final = 0x00000200
SF_NOTIFY_END_OF_REQUEST: Final = 0x00000080
SF_NOTIFY_END_OF_NET_SESSION: Final = 0x00000100
SF_NOTIFY_ORDER_HIGH: Final = 0x00080000
SF_NOTIFY_ORDER_MEDIUM: Final = 0x00040000
SF_NOTIFY_ORDER_LOW: Final = 0x00020000
SF_NOTIFY_ORDER_DEFAULT: Final = SF_NOTIFY_ORDER_LOW
SF_NOTIFY_ORDER_MASK: Final = 917504
SF_STATUS_REQ_FINISHED: Final = 134217728
SF_STATUS_REQ_FINISHED_KEEP_CONN: Final = 134217729
SF_STATUS_REQ_NEXT_NOTIFICATION: Final = 134217730
SF_STATUS_REQ_HANDLED_NOTIFICATION: Final = 134217731
SF_STATUS_REQ_ERROR: Final = 134217732
SF_STATUS_REQ_READ_NEXT: Final = 134217733
HSE_IO_SYNC: Final = 0x00000001
HSE_IO_ASYNC: Final = 0x00000002
HSE_IO_DISCONNECT_AFTER_SEND: Final = 0x00000004
HSE_IO_SEND_HEADERS: Final = 0x00000008
HSE_IO_NODELAY: Final = 0x00001000
HSE_IO_FINAL_SEND: Final = 0x00000010
HSE_IO_CACHE_RESPONSE: Final = 0x00000020
HSE_EXEC_URL_NO_HEADERS: Final = 0x02
HSE_EXEC_URL_IGNORE_CURRENT_INTERCEPTOR: Final = 0x04
HSE_EXEC_URL_IGNORE_VALIDATION_AND_RANGE: Final = 0x10
HSE_EXEC_URL_DISABLE_CUSTOM_ERROR: Final = 0x20
HSE_EXEC_URL_SSI_CMD: Final = 0x40
HSE_EXEC_URL_HTTP_CACHE_ELIGIBLE: Final = 0x80



import re
from _typeshed import Incomplete
from collections.abc import Callable
from typing import ClassVar, Final, NoReturn, Protocol, type_check_only

from docutils import nodes, writers
from docutils.frontend import Values
from docutils.languages import _LanguageModule

@type_check_only
class _RegexPatternSub(Protocol):
    # Matches the signature of the bound instance method `re.Pattern[str].sub` exactly
    def __call__(self, /, repl: str | Callable[[re.Match[str]], str], string: str, count: int = 0) -> str: ...

__docformat__: Final = "reStructuredText"
FIELD_LIST_INDENT: Final[int]
DEFINITION_LIST_INDENT: Final[int]
OPTION_LIST_INDENT: Final[int]
BLOCKQOUTE_INDENT: Final[float]
LITERAL_BLOCK_INDENT: Final[float]
MACRO_DEF: Final[str]
NONPRINTING_BREAKPOINT: Final[str]
NONBREAKING_INSERT_RE: Final[re.Pattern[str]]
NONBREAKING_INSERT_RE2: Final[re.Pattern[str]]

def insert_URI_breakpoints(s: str) -> str: ...

class Writer(writers.Writer[str]):
    translator_class: type[Translator]

class Table:
    def __init__(self) -> None: ...
    def new_row(self) -> None: ...
    def append_separator(self, separator) -> None: ...
    def append_cell(self, cell_lines: list[str]) -> None: ...
    def as_list(self) -> list[str]: ...

class Translator(nodes.NodeVisitor):
    words_and_spaces: ClassVar[re.Pattern[str]]
    possibly_a_roff_command: ClassVar[re.Pattern[str]]
    document_start: ClassVar[str]
    settings: Values
    language: _LanguageModule
    head: list[str]
    body: list[str]
    foot: list[str]
    section_level: int
    context: list[str | int]
    topic_class: str
    colspecs: list[nodes.colspec]
    compact_p: int
    compact_simple: Incomplete
    header_written: int
    authors: list[Incomplete]
    defs: dict[str, tuple[str, ...]]
    def comment_begin(self, text: str) -> str: ...
    def comment(self, text: str) -> str: ...
    def ensure_eol(self) -> None: ...
    def ensure_c_eol(self) -> None: ...
    def astext(self) -> str: ...
    def deunicode(self, text: str) -> str: ...
    def encode_special_chars(self, text: str) -> str: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def list_start(self, node) -> None: ...
    def list_end(self) -> None: ...
    def header(self) -> str: ...
    def append_header(self) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.admonition, name: str | None = None) -> None: ...
    def depart_admonition(self, node: nodes.admonition) -> None: ...
    def visit_attention(self, node: nodes.attention) -> None: ...
    def depart_attention(self, node: nodes.attention) -> None: ...
    def visit_docinfo_item(self, node: nodes.docinfo, name: str) -> None: ...
    def depart_docinfo_item(self, node: nodes.docinfo) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_caution(self, node: nodes.caution) -> None: ...
    def depart_caution(self, node: nodes.caution) -> None: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def write_colspecs(self) -> None: ...
    def visit_comment(self, node: nodes.comment, sub: _RegexPatternSub = ...) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def depart_contact(self, node: nodes.contact) -> None: ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def visit_danger(self, node: nodes.danger) -> None: ...
    def depart_danger(self, node: nodes.danger) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def depart_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_error(self, node: nodes.error) -> None: ...
    def depart_error(self, node: nodes.error) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def footnote_backrefs(self, node) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_hint(self, node: nodes.hint) -> None: ...
    def depart_hint(self, node: nodes.hint) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def visit_important(self, node: nodes.important) -> None: ...
    def depart_important(self, node: nodes.important) -> None: ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_math(self, node: nodes.math) -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_note(self, node: nodes.note) -> None: ...
    def depart_note(self, node: nodes.note) -> None: ...
    def indent(self, by: float = 0.5) -> None: ...
    def dedent(self) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def first_child(self, node): ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    def depart_revision(self, node: nodes.revision) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_section(self, node: nodes.section) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def depart_status(self, node: nodes.status) -> None: ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_substitution_reference(self, node: nodes.substitution_reference) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def visit_tip(self, node: nodes.tip) -> None: ...
    def depart_tip(self, node: nodes.tip) -> None: ...
    def visit_title(self, node: nodes.title) -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def visit_warning(self, node: nodes.warning) -> None: ...
    def depart_warning(self, node: nodes.warning) -> None: ...
    def unimplemented_visit(self, node: nodes.Node) -> NoReturn: ...



from _typeshed import Incomplete

class _YStrip:
    y0: Incomplete
    y1: Incomplete
    slope: Incomplete
    fillColor: Incomplete
    fillColorShaded: Incomplete
    def __init__(self, y0, y1, slope, fillColor, fillColorShaded, shading: float = 0.1) -> None: ...

def mod_2pi(radians): ...

class _Segment:
    a: Incomplete
    b: Incomplete
    x0: Incomplete
    x1: Incomplete
    y0: Incomplete
    y1: Incomplete
    series: Incomplete
    i: Incomplete
    s: Incomplete
    def __init__(self, s, i, data) -> None: ...
    def intersect(self, o, I): ...

def find_intersections(data, small: int = 0): ...



from typing import Any

from asgiref.testing import ApplicationCommunicator as BaseApplicationCommunicator

def no_op() -> None: ...

class ApplicationCommunicator(BaseApplicationCommunicator):
    # ASGI messages are dictionaries with a "type" key and protocol-specific fields.
    # Dictionary values can be strings, bytes, lists, or other types depending on the protocol:
    # - HTTP: {"type": "http.request", "body": b"request data", "headers": [...], ...}
    # - WebSocket: {"type": "websocket.receive", "bytes": b"binary data"} or {"text": "string"}
    # - Custom protocols: Application-specific message dictionaries
    async def send_input(self, message: dict[str, Any]) -> None: ...
    async def receive_output(self, timeout: float = 1) -> dict[str, Any]: ...

    # The following methods are not present in the original source code,
    # but are commonly used in practice. Since the base package doesn't
    # provide type hints for them, they are added here to improve type correctness.
    async def receive_nothing(self, timeout: float = 0.1, interval: float = 0.01) -> bool: ...
    async def wait(self, timeout: float = 1) -> None: ...
    def stop(self, exceptions: bool = True) -> None: ...



from collections.abc import Callable, Mapping, Sequence
from typing import Any, Final
from typing_extensions import TypeAlias, deprecated

import docutils.parsers.rst.states
from docutils import nodes
from docutils.languages import _LanguageModule
from docutils.nodes import Node, system_message
from docutils.parsers.rst.states import Inliner
from docutils.utils import Reporter

__docformat__: Final = "reStructuredText"
DEFAULT_INTERPRETED_ROLE: Final = "title-reference"

_RoleFn: TypeAlias = Callable[
    [str, str, str, int, docutils.parsers.rst.states.Inliner, Mapping[str, Any], Sequence[str]],
    tuple[Sequence[nodes.reference], Sequence[nodes.reference]],
]

def register_canonical_role(name: str, role_fn: _RoleFn) -> None: ...
def register_local_role(name: str, role_fn: _RoleFn) -> None: ...
def role(
    role_name: str, language_module: _LanguageModule, lineno: int, reporter: Reporter
) -> tuple[_RoleFn | None, list[system_message]]: ...
def set_implicit_options(role_fn: _RoleFn) -> None: ...
def register_generic_role(canonical_name: str, node_class: type[Node]) -> None: ...

class GenericRole:
    name: str
    node_class: type[Node]
    def __init__(self, role_name: str, node_class: type[Node]) -> None: ...
    def __call__(
        self,
        role: str,
        rawtext: str,
        text: str,
        lineno: int,
        inliner: Inliner,
        options: Mapping[str, Any] | None = None,
        content: Sequence[str] | None = None,
    ) -> tuple[list[Node], list[system_message]]: ...

class CustomRole:
    name: str
    base_role: _RoleFn | CustomRole
    options: Mapping[str, Any]
    content: Sequence[str]
    supplied_options: Mapping[str, Any]
    supplied_content: Sequence[str]
    def __init__(
        self,
        role_name: str,
        base_role: _RoleFn | CustomRole,
        options: Mapping[str, Any] | None = None,
        content: Sequence[str] | None = None,
    ) -> None: ...
    def __call__(
        self,
        role: str,
        rawtext: str,
        text: str,
        lineno: int,
        inliner: Inliner,
        options: Mapping[str, Any] | None = None,
        content: Sequence[str] | None = None,
    ) -> tuple[list[Node], list[system_message]]: ...

def generic_custom_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def pep_reference_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def rfc_reference_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def raw_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def code_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def math_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
def unimplemented_role(
    role: str,
    rawtext: str,
    text: str,
    lineno: int,
    inliner: Inliner,
    options: Mapping[str, Any] | None = None,
    content: Sequence[str] | None = None,
) -> tuple[list[Node], list[system_message]]: ...
@deprecated("Deprecated and will be removed in Docutils 2.0, Use `roles.normalize_options()` instead.")
def set_classes(options: dict[str, str]) -> None: ...
@deprecated("Deprecated and will be removed in Docutils 2.0, Use `roles.normalize_options()` instead.")
def normalized_role_options(options: Mapping[str, Any] | None) -> dict[str, Any]: ...
def normalize_options(options: Mapping[str, Any] | None) -> dict[str, Any]: ...



from typing import Any

conf_ldif_line_length: Any

def safe_ldif_string(bytes_value): ...
def add_controls(controls, all_base64): ...
def add_attributes(attributes, all_base64): ...
def sort_ldif_lines(lines, sort_order): ...
def search_response_to_ldif(entries, all_base64, sort_order=None): ...
def add_request_to_ldif(entry, all_base64, sort_order=None): ...
def delete_request_to_ldif(entry, all_base64, sort_order=None): ...
def modify_request_to_ldif(entry, all_base64, sort_order=None): ...
def modify_dn_request_to_ldif(entry, all_base64, sort_order=None): ...
def operation_to_ldif(operation_type, entries, all_base64: bool = False, sort_order=None): ...
def add_ldif_header(ldif_lines): ...
def ldif_sort(line, sort_order): ...
def decode_persistent_search_control(change): ...
def persistent_search_response_to_ldif(change): ...



from typing import Any, TypeVar

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

class PilNotAvailable(ImportError): ...
class FontNotFound(Exception): ...

class FontManager:
    font_name: Any
    font_size: Any
    fonts: Any
    encoding: Any
    variable: bool
    def __init__(self, font_name, font_size: int = 14) -> None: ...
    def get_char_size(self): ...
    def get_text_size(self, text): ...
    def get_font(self, bold, oblique): ...
    def get_style(self, style): ...

class ImageFormatter(Formatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    unicodeoutput: bool
    default_image_format: str
    encoding: str
    styles: Any
    background_color: str
    image_format: Any
    image_pad: Any
    line_pad: Any
    fonts: Any
    line_number_fg: Any
    line_number_bg: Any
    line_number_chars: Any
    line_number_bold: Any
    line_number_italic: Any
    line_number_pad: Any
    line_numbers: Any
    line_number_separator: Any
    line_number_step: Any
    line_number_start: Any
    line_number_width: Any
    hl_lines: Any
    hl_color: Any
    drawables: Any
    def get_style_defs(self, arg: str = "") -> None: ...
    def format(self, tokensource, outfile) -> None: ...

class GifImageFormatter(ImageFormatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    default_image_format: str

class JpgImageFormatter(ImageFormatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    default_image_format: str

class BmpImageFormatter(ImageFormatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    default_image_format: str



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["cd_index"]

@_dispatchable
def cd_index(G: Graph[_Node], node: _Node, time_delta, *, time: str = "time", weight: str | None = None): ...



from collections.abc import Collection

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["laplacian_centrality"]

@_dispatchable
def laplacian_centrality(
    G: Graph[_Node],
    normalized: bool = True,
    nodelist: Collection[_Node] | None = None,
    weight: str | None = "weight",
    walk_type: str | None = None,
    alpha: float = 0.95,
): ...



from typing import Final

__version__: Final[str]

def makeA85Image(filename, IMG=None, detectJpeg: bool = False): ...
def makeRawImage(filename, IMG=None, detectJpeg: bool = False): ...
def cacheImageFile(filename, returnInMemory: int = 0, IMG=None): ...
def preProcessImages(spec) -> None: ...
def cachedImageExists(filename): ...
def readJPEGInfo(image): ...

class _fusc:
    def __init__(self, k, n) -> None: ...
    def encrypt(self, s): ...
    def decrypt(self, s): ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_pb2
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class CompressedComponentMetadata(google.protobuf.message.Message):
    """This file contains protocol buffers for working with tf.data Datasets.

    Metadata describing a compressed component of a dataset element.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    TENSOR_SHAPE_FIELD_NUMBER: builtins.int
    UNCOMPRESSED_BYTES_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    """The dtype of the component tensor."""
    @property
    def tensor_shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
        """The shape of the component tensor."""

    @property
    def uncompressed_bytes(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """The amount of uncompressed tensor data.
        - For string tensors, there is an element for each string indicating the
        size of the string.
        - For all other tensors, there is a single element indicating the size of
        the tensor.
        """

    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        tensor_shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        uncompressed_bytes: collections.abc.Iterable[builtins.int] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["tensor_shape", b"tensor_shape"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "dtype", b"dtype", "tensor_shape", b"tensor_shape", "uncompressed_bytes", b"uncompressed_bytes"
        ],
    ) -> None: ...

global___CompressedComponentMetadata = CompressedComponentMetadata

@typing.final
class CompressedElement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    COMPONENT_METADATA_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    """Compressed tensor bytes for all components of the element."""
    version: builtins.int
    """Version of the CompressedElement. CompressedElements may be stored on disk
    and read back by later versions of code, so we store a version number to
    help readers understand which version they are reading. When you add a new
    field to this proto, you need to increment kCompressedElementVersion in
    tensorflow/core/data/compression_utils.cc.
    """
    @property
    def component_metadata(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___CompressedComponentMetadata]:
        """Metadata for the components of the element."""

    def __init__(
        self,
        *,
        data: builtins.bytes | None = ...,
        component_metadata: collections.abc.Iterable[global___CompressedComponentMetadata] | None = ...,
        version: builtins.int | None = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["component_metadata", b"component_metadata", "data", b"data", "version", b"version"]
    ) -> None: ...

global___CompressedElement = CompressedElement

@typing.final
class UncompressedElement(google.protobuf.message.Message):
    """An uncompressed dataset element."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMPONENTS_FIELD_NUMBER: builtins.int
    @property
    def components(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.framework.tensor_pb2.TensorProto
    ]: ...
    def __init__(
        self, *, components: collections.abc.Iterable[tensorflow.core.framework.tensor_pb2.TensorProto] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["components", b"components"]) -> None: ...

global___UncompressedElement = UncompressedElement



from _typeshed import Incomplete
from abc import ABCMeta
from collections.abc import Mapping

class Trie(Mapping[Incomplete, Incomplete], metaclass=ABCMeta):
    def keys(self, prefix=None): ...
    def has_keys_with_prefix(self, prefix): ...
    def longest_prefix(self, prefix): ...
    def longest_prefix_item(self, prefix): ...



from logging import Logger

from authlib.integrations.base_client.sync_app import OAuth1Base, OAuth2Base

log: Logger

__all__ = ["AsyncOAuth1Mixin", "AsyncOAuth2Mixin"]

class AsyncOAuth1Mixin(OAuth1Base):
    async def request(self, method, url, token=None, **kwargs): ...
    async def create_authorization_url(self, redirect_uri=None, **kwargs): ...
    async def fetch_access_token(self, request_token=None, **kwargs): ...

class AsyncOAuth2Mixin(OAuth2Base):
    async def load_server_metadata(self): ...
    async def request(self, method, url, token=None, **kwargs): ...
    async def create_authorization_url(self, redirect_uri=None, **kwargs): ...
    async def fetch_access_token(self, redirect_uri=None, **kwargs): ...



from _typeshed import SupportsWrite

from ..extractor.common import _InfoDict
from .common import FileDownloader

class FFmpegSinkFD(FileDownloader):
    def real_download(self, filename: str, info_dict: _InfoDict) -> bool: ...
    async def real_connection(self, sink: SupportsWrite[bytes], info_dict: _InfoDict) -> None: ...

class WebSocketFragmentFD(FFmpegSinkFD):
    async def real_connection(self, sink: SupportsWrite[bytes], info_dict: _InfoDict) -> None: ...



import sys
from _typeshed import FileDescriptor, StrOrBytesPath
from collections.abc import Callable
from types import TracebackType
from typing import Any, Literal, Protocol, overload
from typing_extensions import TypeAlias, TypeVarTuple, Unpack

_Ts = TypeVarTuple("_Ts")

# gevent uses zope.interface interanlly which does not work well with type checkers
# partially due to the odd call signatures without self and partially due to them
# behaving essentially like a Protocol, so a mypy plugin is necessary to type check
# them correctly, and then you still have to give up on some valuable features due
# to the missing self/cls argument.
# To ensure maximum compatibility with other type checkers and so we don't depend
# on mypy-zope we define an equivalent Protocol for each interface, which we will
# use on arguments in place of the interface
# it also looks like ILoop is possibly too strict, since there are additional
# properties and methods that are available on all event loops, so these have
# been added as well, instead of completely mirroring the internal interface

class _Loop(Protocol):  # noqa: Y046
    @property
    def approx_timer_resolution(self) -> float: ...
    @property
    def default(self) -> bool: ...
    @property
    def iteration(self) -> int: ...
    @property
    def depth(self) -> int: ...
    @property
    def backend_int(self) -> int: ...
    @property
    def backend(self) -> str | int: ...
    @property
    def pendingcnt(self) -> int: ...
    @property
    def activecnt(self) -> int: ...
    def handle_error(
        self, context: object | None, type: type[BaseException] | None, value: BaseException | None, tb: TracebackType | None
    ) -> None: ...
    def run(self, nowait: bool = False, once: bool = False) -> None: ...
    def reinit(self) -> None: ...
    def ref(self) -> None: ...
    def unref(self) -> None: ...
    def break_(self, how: int | None = ...) -> None: ...
    def verify(self) -> None: ...
    def now(self) -> float: ...
    def update_now(self) -> None: ...
    def destroy(self) -> None: ...
    def io(self, fd: FileDescriptor, events: int, ref: bool = True, priority: int | None = None) -> _IoWatcher: ...
    def closing_fd(self, fd: FileDescriptor) -> bool: ...
    def timer(self, after: float, repeat: float = 0.0, ref: bool = True, priority: int | None = None) -> _TimerWatcher: ...
    def signal(self, signum: int, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def idle(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def prepare(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    def check(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
    if sys.platform != "win32":
        def fork(self, ref: bool = True, priority: int | None = None) -> _Watcher: ...
        def child(self, pid: int, trace: int = 0, ref: bool = True) -> _ChildWatcher: ...
        def install_sigchld(self) -> None: ...

    def async_(self, ref: bool = True, priority: int | None = None) -> _AsyncWatcher: ...
    def stat(self, path: str, interval: float = 0.0, ref: bool = True, priority: bool | None = ...) -> _StatWatcher: ...
    def run_callback(self, func: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> _Callback: ...
    def run_callback_threadsafe(self, func: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> _Callback: ...
    def fileno(self) -> FileDescriptor | None: ...

class _Watcher(Protocol):
    # while IWatcher allows for kwargs the actual implementation does not...
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...
    def stop(self) -> None: ...
    def close(self) -> None: ...

# this matches Intersection[_Watcher, TimerMixin]
class _TimerWatcher(_Watcher, Protocol):
    # this has one specific allowed keyword argument, if it is given we don't try to check
    # the passed in arguments, but if it isn't passed in, then we do.
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts], update: bool) -> None: ...
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...
    @overload
    def again(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts], update: bool) -> None: ...
    @overload
    def again(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...

# this matches Intersection[_Watcher, IoMixin]
class _IoWatcher(_Watcher, Protocol):
    EVENT_MASK: int
    # pass_events means the first argument of the callback needs to be an integer, but we can't
    # type check the other passed in args in this case
    @overload
    def start(self, callback: Callable[[int, Unpack[_Ts]], Any], *args: Unpack[_Ts], pass_events: Literal[True]) -> None: ...
    @overload
    def start(self, callback: Callable[[Unpack[_Ts]], Any], *args: Unpack[_Ts]) -> None: ...

# this matches Intersection[_Watcher, ChildMixin]
class _ChildWatcher(_Watcher, Protocol):
    @property
    def pid(self) -> int: ...
    @property
    def rpid(self) -> int | None: ...
    @property
    def rstatus(self) -> int: ...

# this matches Intersection[_Watcher, AsyncMixin]
class _AsyncWatcher(_Watcher, Protocol):
    def send(self) -> None: ...
    def send_ignoring_arg(self, ignored: object, /) -> None: ...
    @property
    def pending(self) -> bool: ...

# all implementations return something of this shape
class _StatResult(Protocol):
    @property
    def st_nlink(self) -> int: ...

# this matches Intersection[_Watcher, StatMixin]
class _StatWatcher(_Watcher, Protocol):
    @property
    def path(self) -> StrOrBytesPath: ...
    @property
    def attr(self) -> _StatResult | None: ...
    @property
    def prev(self) -> _StatResult | None: ...
    @property
    def interval(self) -> float: ...

class _Callback(Protocol):
    pending: bool
    def stop(self) -> None: ...
    def close(self) -> None: ...

_FullSockAddr: TypeAlias = tuple[str, int, int, int]  # host, port, flowinfo, scopeid
_SockAddr: TypeAlias = _FullSockAddr | tuple[str, int]
_AddrinfoResult: TypeAlias = list[tuple[int, int, int, str, _SockAddr]]  # family, type, protocol, cname, sockaddr
_NameinfoResult: TypeAlias = tuple[str, str]

class _Resolver(Protocol):  # noqa: Y046
    def close(self) -> None: ...
    def gethostbyname(self, hostname: str, family: int = 2) -> str: ...
    def gethostbyname_ex(self, hostname: str, family: int = 2) -> tuple[str, list[str], list[str]]: ...
    def getaddrinfo(
        self, host: str, port: int, family: int = 0, socktype: int = 0, proto: int = 0, flags: int = 0
    ) -> _AddrinfoResult: ...
    def gethostbyaddr(self, ip_address: str) -> tuple[str, list[str], list[str]]: ...
    def getnameinfo(self, sockaddr: _SockAddr, flags: int) -> _NameinfoResult: ...



from _typeshed import Unused
from collections.abc import Callable

from oauthlib.common import Request
from oauthlib.oauth2.rfc6749.endpoints import (
    AuthorizationEndpoint,
    IntrospectEndpoint,
    ResourceEndpoint,
    RevocationEndpoint,
    TokenEndpoint,
)
from oauthlib.oauth2.rfc6749.grant_types import (
    AuthorizationCodeGrant as OAuth2AuthorizationCodeGrant,
    ClientCredentialsGrant,
    ImplicitGrant as OAuth2ImplicitGrant,
    RefreshTokenGrant,
    ResourceOwnerPasswordCredentialsGrant,
)
from oauthlib.oauth2.rfc6749.request_validator import RequestValidator as OAuth2RequestValidator
from oauthlib.oauth2.rfc6749.tokens import BearerToken
from oauthlib.oauth2.rfc8628.grant_types import DeviceCodeGrant

from ..grant_types import AuthorizationCodeGrant, HybridGrant, ImplicitGrant
from ..grant_types.dispatchers import (
    AuthorizationCodeGrantDispatcher,
    AuthorizationTokenGrantDispatcher,
    ImplicitTokenGrantDispatcher,
)
from ..tokens import JWTToken
from .userinfo import UserInfoEndpoint

class Server(AuthorizationEndpoint, IntrospectEndpoint, TokenEndpoint, ResourceEndpoint, RevocationEndpoint, UserInfoEndpoint):
    auth_grant: OAuth2AuthorizationCodeGrant
    implicit_grant: OAuth2ImplicitGrant
    password_grant: ResourceOwnerPasswordCredentialsGrant
    credentials_grant: ClientCredentialsGrant
    refresh_grant: RefreshTokenGrant
    openid_connect_auth: AuthorizationCodeGrant
    openid_connect_implicit: ImplicitGrant
    openid_connect_hybrid: HybridGrant
    device_code_grant: DeviceCodeGrant
    bearer: BearerToken
    jwt: JWTToken
    auth_grant_choice: AuthorizationCodeGrantDispatcher
    implicit_grant_choice: ImplicitTokenGrantDispatcher
    token_grant_choice: AuthorizationTokenGrantDispatcher
    def __init__(
        self,
        request_validator: OAuth2RequestValidator,
        token_expires_in: int | Callable[[Request], int] | None = None,
        token_generator: Callable[[Request], str] | None = None,
        refresh_token_generator: Callable[[Request], str] | None = None,
        *args: Unused,
    ) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import sys
import typing

import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Race:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _RaceEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Race.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NoRace: _Race.ValueType  # 0
    Terran: _Race.ValueType  # 1
    Zerg: _Race.ValueType  # 2
    Protoss: _Race.ValueType  # 3
    Random: _Race.ValueType  # 4

class Race(_Race, metaclass=_RaceEnumTypeWrapper): ...

NoRace: Race.ValueType  # 0
Terran: Race.ValueType  # 1
Zerg: Race.ValueType  # 2
Protoss: Race.ValueType  # 3
Random: Race.ValueType  # 4
global___Race = Race

@typing.final
class AvailableAbility(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ABILITY_ID_FIELD_NUMBER: builtins.int
    REQUIRES_POINT_FIELD_NUMBER: builtins.int
    ability_id: builtins.int
    requires_point: builtins.bool
    def __init__(self, *, ability_id: builtins.int | None = ..., requires_point: builtins.bool | None = ...) -> None: ...
    def HasField(
        self, field_name: typing.Literal["ability_id", b"ability_id", "requires_point", b"requires_point"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["ability_id", b"ability_id", "requires_point", b"requires_point"]
    ) -> None: ...

global___AvailableAbility = AvailableAbility

@typing.final
class ImageData(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BITS_PER_PIXEL_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    bits_per_pixel: builtins.int
    """Number of bits per pixel; 8 bits for a byte etc."""
    data: builtins.bytes
    """Binary data; the size of this buffer in bytes is width * height * bits_per_pixel / 8."""
    @property
    def size(self) -> global___Size2DI:
        """Dimension in pixels."""

    def __init__(
        self, *, bits_per_pixel: builtins.int | None = ..., size: global___Size2DI | None = ..., data: builtins.bytes | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["bits_per_pixel", b"bits_per_pixel", "data", b"data", "size", b"size"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["bits_per_pixel", b"bits_per_pixel", "data", b"data", "size", b"size"]
    ) -> None: ...

global___ImageData = ImageData

@typing.final
class PointI(google.protobuf.message.Message):
    """Point on the screen/minimap (e.g., 0..64).
    Note: bottom left of the screen is 0, 0.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.int
    y: builtins.int
    def __init__(self, *, x: builtins.int | None = ..., y: builtins.int | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___PointI = PointI

@typing.final
class RectangleI(google.protobuf.message.Message):
    """Screen space rectangular area."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    P0_FIELD_NUMBER: builtins.int
    P1_FIELD_NUMBER: builtins.int
    @property
    def p0(self) -> global___PointI: ...
    @property
    def p1(self) -> global___PointI: ...
    def __init__(self, *, p0: global___PointI | None = ..., p1: global___PointI | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["p0", b"p0", "p1", b"p1"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["p0", b"p0", "p1", b"p1"]) -> None: ...

global___RectangleI = RectangleI

@typing.final
class Point2D(google.protobuf.message.Message):
    """Point on the game board, 0..255.
    Note: bottom left of the screen is 0, 0.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    def __init__(self, *, x: builtins.float | None = ..., y: builtins.float | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___Point2D = Point2D

@typing.final
class Point(google.protobuf.message.Message):
    """Point on the game board, 0..255.
    Note: bottom left of the screen is 0, 0.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self, *, x: builtins.float | None = ..., y: builtins.float | None = ..., z: builtins.float | None = ...
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___Point = Point

@typing.final
class Size2DI(google.protobuf.message.Message):
    """Screen dimensions."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    x: builtins.int
    y: builtins.int
    def __init__(self, *, x: builtins.int | None = ..., y: builtins.int | None = ...) -> None: ...
    def HasField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y"]) -> None: ...

global___Size2DI = Size2DI



from oauthlib.oauth2.rfc6749.errors import OAuth2Error

class AuthorizationPendingError(OAuth2Error):
    error: str

class SlowDownError(OAuth2Error):
    error: str

class ExpiredTokenError(OAuth2Error):
    error: str

class AccessDenied(OAuth2Error):
    error: str



from collections.abc import Iterable, Iterator
from typing import Any, Generic, TypeVar

_T = TypeVar("_T", default=Any)

class Filter(Generic[_T]):
    source: Iterable[_T]
    def __init__(self, source: Iterable[_T]) -> None: ...
    def __iter__(self) -> Iterator[_T]: ...
    def __getattr__(self, name: str) -> Any: ...  # Depends on `source`



from wtforms.widgets.core import (
    CheckboxInput as CheckboxInput,
    ColorInput as ColorInput,
    DateInput as DateInput,
    DateTimeInput as DateTimeInput,
    DateTimeLocalInput as DateTimeLocalInput,
    EmailInput as EmailInput,
    FileInput as FileInput,
    HiddenInput as HiddenInput,
    Input as Input,
    ListWidget as ListWidget,
    MonthInput as MonthInput,
    NumberInput as NumberInput,
    Option as Option,
    PasswordInput as PasswordInput,
    RadioInput as RadioInput,
    RangeInput as RangeInput,
    SearchInput as SearchInput,
    Select as Select,
    SubmitInput as SubmitInput,
    TableWidget as TableWidget,
    TelInput as TelInput,
    TextArea as TextArea,
    TextInput as TextInput,
    TimeInput as TimeInput,
    URLInput as URLInput,
    WeekInput as WeekInput,
    html_params as html_params,
)

__all__ = [
    "CheckboxInput",
    "ColorInput",
    "DateInput",
    "DateTimeInput",
    "DateTimeLocalInput",
    "EmailInput",
    "FileInput",
    "HiddenInput",
    "ListWidget",
    "MonthInput",
    "NumberInput",
    "Option",
    "PasswordInput",
    "RadioInput",
    "RangeInput",
    "SearchInput",
    "Select",
    "SubmitInput",
    "TableWidget",
    "TextArea",
    "TextInput",
    "TelInput",
    "TimeInput",
    "URLInput",
    "WeekInput",
    "html_params",
    "Input",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import google.protobuf.descriptor
from tensorflow.compiler.xla.tsl.protobuf.bfc_memory_map_pb2 import (
    BinSummary as BinSummary,
    MemAllocatorStats as MemAllocatorStats,
    MemChunk as MemChunk,
    MemoryDump as MemoryDump,
    SnapShot as SnapShot,
)

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor



from typing import ClassVar

from .common import InfoExtractor

class CommonMistakesIE(InfoExtractor):
    IE_DESC: ClassVar[bool]

class UnicodeBOMIE(InfoExtractor):
    IE_DESC: ClassVar[bool]

class BlobIE(InfoExtractor):
    IE_DESC: ClassVar[bool]



from collections.abc import AsyncIterable

from grpc_reflection.v1alpha import reflection_pb2
from grpc_reflection.v1alpha._base import BaseReflectionServicer

class ReflectionServicer(BaseReflectionServicer):
    async def ServerReflectionInfo(
        self, request_iterator: AsyncIterable[reflection_pb2.ServerReflectionRequest], unused_context
    ) -> AsyncIterable[reflection_pb2.ServerReflectionResponse]: ...

__all__ = ["ReflectionServicer"]



from _typeshed import Incomplete

from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNConfig import LexerATNConfig as LexerATNConfig
from antlr4.atn.ATNConfigSet import ATNConfigSet as ATNConfigSet, OrderedATNConfigSet as OrderedATNConfigSet
from antlr4.atn.ATNSimulator import ATNSimulator as ATNSimulator
from antlr4.atn.ATNState import ATNState as ATNState, RuleStopState as RuleStopState
from antlr4.atn.LexerActionExecutor import LexerActionExecutor as LexerActionExecutor
from antlr4.atn.Transition import Transition as Transition
from antlr4.dfa.DFA import DFA
from antlr4.dfa.DFAState import DFAState as DFAState
from antlr4.error.Errors import (
    LexerNoViableAltException as LexerNoViableAltException,
    UnsupportedOperationException as UnsupportedOperationException,
)
from antlr4.InputStream import InputStream as InputStream
from antlr4.PredictionContext import (
    PredictionContext as PredictionContext,
    PredictionContextCache as PredictionContextCache,
    SingletonPredictionContext as SingletonPredictionContext,
)
from antlr4.Token import Token as Token

class SimState:
    __slots__ = ("index", "line", "column", "dfaState")
    def __init__(self) -> None: ...
    index: int
    line: int
    column: int
    dfaState: Incomplete
    def reset(self) -> None: ...

class LexerATNSimulator(ATNSimulator):
    __slots__ = ("decisionToDFA", "recog", "startIndex", "line", "column", "mode", "DEFAULT_MODE", "MAX_CHAR_VALUE", "prevAccept")
    debug: bool
    dfa_debug: bool
    MIN_DFA_EDGE: int
    MAX_DFA_EDGE: int
    ERROR: Incomplete
    decisionToDFA: Incomplete
    recog: Incomplete
    startIndex: int
    line: int
    column: int
    mode: Incomplete
    DEFAULT_MODE: Incomplete
    MAX_CHAR_VALUE: Incomplete
    prevAccept: Incomplete
    def __init__(self, recog, atn: ATN, decisionToDFA: list[DFA], sharedContextCache: PredictionContextCache) -> None: ...
    def copyState(self, simulator: LexerATNSimulator): ...
    def match(self, input: InputStream, mode: int): ...
    def reset(self) -> None: ...
    def matchATN(self, input: InputStream): ...
    def execATN(self, input: InputStream, ds0: DFAState): ...
    def getExistingTargetState(self, s: DFAState, t: int): ...
    def computeTargetState(self, input: InputStream, s: DFAState, t: int): ...
    def failOrAccept(self, prevAccept: SimState, input: InputStream, reach: ATNConfigSet, t: int): ...
    def getReachableConfigSet(self, input: InputStream, closure: ATNConfigSet, reach: ATNConfigSet, t: int): ...
    def accept(
        self, input: InputStream, lexerActionExecutor: LexerActionExecutor, startIndex: int, index: int, line: int, charPos: int
    ): ...
    def getReachableTarget(self, trans: Transition, t: int): ...
    def computeStartState(self, input: InputStream, p: ATNState): ...
    def closure(
        self,
        input: InputStream,
        config: LexerATNConfig,
        configs: ATNConfigSet,
        currentAltReachedAcceptState: bool,
        speculative: bool,
        treatEofAsEpsilon: bool,
    ): ...
    def getEpsilonTarget(
        self,
        input: InputStream,
        config: LexerATNConfig,
        t: Transition,
        configs: ATNConfigSet,
        speculative: bool,
        treatEofAsEpsilon: bool,
    ): ...
    def evaluatePredicate(self, input: InputStream, ruleIndex: int, predIndex: int, speculative: bool): ...
    def captureSimState(self, settings: SimState, input: InputStream, dfaState: DFAState): ...
    def addDFAEdge(self, from_: DFAState, tk: int, to: DFAState | None = None, cfgs: ATNConfigSet | None = None) -> DFAState: ...
    def addDFAState(self, configs: ATNConfigSet) -> DFAState: ...
    def getDFA(self, mode: int): ...
    def getText(self, input: InputStream): ...
    def consume(self, input: InputStream): ...
    def getTokenName(self, t: int): ...



# Alias the import to avoid name clash with a class called "Final"
from typing import Final as _Final

from pyasn1.type.constraint import ConstraintsIntersection, SingleValueConstraint, ValueRangeConstraint, ValueSizeConstraint
from pyasn1.type.namedtype import NamedTypes
from pyasn1.type.namedval import NamedValues
from pyasn1.type.tag import TagSet
from pyasn1.type.univ import Boolean, Choice, Enumerated, Integer, Null, OctetString, Sequence, SequenceOf, SetOf

LDAP_MAX_INT: _Final[int]
MAXINT: _Final[Integer]
rangeInt0ToMaxConstraint: ValueRangeConstraint
rangeInt1To127Constraint: ValueRangeConstraint
size1ToMaxConstraint: ValueSizeConstraint
responseValueConstraint: SingleValueConstraint
# Custom constraints. They have yet to be implemented so ldap3 keeps them as None.
numericOIDConstraint: None
distinguishedNameConstraint: None
nameComponentConstraint: None
attributeDescriptionConstraint: None
uriConstraint: None
attributeSelectorConstraint: None

class Integer0ToMax(Integer):
    subtypeSpec: ConstraintsIntersection

class LDAPString(OctetString):
    encoding: str

class MessageID(Integer0ToMax): ...
class LDAPOID(OctetString): ...
class LDAPDN(LDAPString): ...
class RelativeLDAPDN(LDAPString): ...
class AttributeDescription(LDAPString): ...

class AttributeValue(OctetString):
    encoding: str

class AssertionValue(OctetString):
    encoding: str

class AttributeValueAssertion(Sequence):
    componentType: NamedTypes

class MatchingRuleId(LDAPString): ...

class Vals(SetOf):
    componentType: AttributeValue  # type: ignore[assignment]

class ValsAtLeast1(SetOf):
    componentType: AttributeValue  # type: ignore[assignment]
    subtypeSpec: ConstraintsIntersection

class PartialAttribute(Sequence):
    componentType: NamedTypes

class Attribute(Sequence):
    componentType: NamedTypes

class AttributeList(SequenceOf):
    componentType: Attribute  # type: ignore[assignment]

class Simple(OctetString):
    tagSet: TagSet
    encoding: str

class Credentials(OctetString):
    encoding: str

class SaslCredentials(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class SicilyPackageDiscovery(OctetString):
    tagSet: TagSet
    encoding: str

class SicilyNegotiate(OctetString):
    tagSet: TagSet
    encoding: str

class SicilyResponse(OctetString):
    tagSet: TagSet
    encoding: str

class AuthenticationChoice(Choice):
    componentType: NamedTypes

class Version(Integer):
    subtypeSpec: ConstraintsIntersection

class ResultCode(Enumerated):
    namedValues: NamedValues
    subTypeSpec: ConstraintsIntersection

class URI(LDAPString): ...

class Referral(SequenceOf):
    tagSet: TagSet
    componentType: URI  # type: ignore[assignment]

class ServerSaslCreds(OctetString):
    tagSet: TagSet
    encoding: str

class LDAPResult(Sequence):
    componentType: NamedTypes

class Criticality(Boolean):
    defaultValue: bool

class ControlValue(OctetString):
    encoding: str

class Control(Sequence):
    componentType: NamedTypes

class Controls(SequenceOf):
    tagSet: TagSet
    componentType: Control  # type: ignore[assignment]

class Scope(Enumerated):
    namedValues: NamedValues

class DerefAliases(Enumerated):
    namedValues: NamedValues

class TypesOnly(Boolean): ...
class Selector(LDAPString): ...

class AttributeSelection(SequenceOf):
    componentType: Selector  # type: ignore[assignment]

class MatchingRule(MatchingRuleId):
    tagSet: TagSet

class Type(AttributeDescription):
    tagSet: TagSet

class MatchValue(AssertionValue):
    tagSet: TagSet

class DnAttributes(Boolean):
    tagSet: TagSet
    defaultValue: Boolean

class MatchingRuleAssertion(Sequence):
    componentType: NamedTypes

class Initial(AssertionValue):
    tagSet: TagSet

class Any(AssertionValue):
    tagSet: TagSet

class Final(AssertionValue):
    tagSet: TagSet

class Substring(Choice):
    componentType: NamedTypes

class Substrings(SequenceOf):
    subtypeSpec: ConstraintsIntersection
    componentType: Substring  # type: ignore[assignment]

class SubstringFilter(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class And(SetOf):
    tagSet: TagSet
    subtypeSpec: ConstraintsIntersection
    componentType: Filter  # type: ignore[assignment]

class Or(SetOf):
    tagSet: TagSet
    subtypeSpec: ConstraintsIntersection
    componentType: Filter  # type: ignore[assignment]

class Not(Choice): ...

class EqualityMatch(AttributeValueAssertion):
    tagSet: TagSet

class GreaterOrEqual(AttributeValueAssertion):
    tagSet: TagSet

class LessOrEqual(AttributeValueAssertion):
    tagSet: TagSet

class Present(AttributeDescription):
    tagSet: TagSet

class ApproxMatch(AttributeValueAssertion):
    tagSet: TagSet

class ExtensibleMatch(MatchingRuleAssertion):
    tagSet: TagSet

class Filter(Choice):
    componentType: NamedTypes

class PartialAttributeList(SequenceOf):
    componentType: PartialAttribute  # type: ignore[assignment]

class Operation(Enumerated):
    namedValues: NamedValues

class Change(Sequence):
    componentType: NamedTypes

class Changes(SequenceOf):
    componentType: Change  # type: ignore[assignment]

class DeleteOldRDN(Boolean): ...

class NewSuperior(LDAPDN):
    tagSet: TagSet

class RequestName(LDAPOID):
    tagSet: TagSet

class RequestValue(OctetString):
    tagSet: TagSet
    encoding: str

class ResponseName(LDAPOID):
    tagSet: TagSet

class ResponseValue(OctetString):
    tagSet: TagSet
    encoding: str

class IntermediateResponseName(LDAPOID):
    tagSet: TagSet

class IntermediateResponseValue(OctetString):
    tagSet: TagSet
    encoding: str

class BindRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class BindResponse(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class UnbindRequest(Null):
    tagSet: TagSet

class SearchRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class SearchResultReference(SequenceOf):
    tagSet: TagSet
    subtypeSpec: ConstraintsIntersection
    componentType: URI  # type: ignore[assignment]

class SearchResultEntry(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class SearchResultDone(LDAPResult):
    tagSet: TagSet

class ModifyRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class ModifyResponse(LDAPResult):
    tagSet: TagSet

class AddRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class AddResponse(LDAPResult):
    tagSet: TagSet

class DelRequest(LDAPDN):
    tagSet: TagSet

class DelResponse(LDAPResult):
    tagSet: TagSet

class ModifyDNRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class ModifyDNResponse(LDAPResult):
    tagSet: TagSet

class CompareRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class CompareResponse(LDAPResult):
    tagSet: TagSet

class AbandonRequest(MessageID):
    tagSet: TagSet

class ExtendedRequest(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class ExtendedResponse(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class IntermediateResponse(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class ProtocolOp(Choice):
    componentType: NamedTypes

class LDAPMessage(Sequence):
    componentType: NamedTypes



from _typeshed import Incomplete
from typing import Final

from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult

class Address(Resource):
    class ShippingMethod:
        SameDay: Final = "same_day"
        NextDay: Final = "next_day"
        Priority: Final = "priority"
        Ground: Final = "ground"
        Electronic: Final = "electronic"
        ShipToStore: Final = "ship_to_store"
        PickupInStore: Final = "pickup_in_store"

    @staticmethod
    def create(params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def delete(customer_id: str, address_id: str) -> SuccessfulResult: ...
    @staticmethod
    def find(customer_id: str, address_id: str) -> Address: ...
    @staticmethod
    def update(
        customer_id: str, address_id: str, params: dict[str, Incomplete] | None = None
    ) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def create_signature() -> list[str | dict[str, list[str]]]: ...
    @staticmethod
    def update_signature() -> list[str | dict[str, list[str]]]: ...



from pyasn1.type.namedtype import NamedTypes
from pyasn1.type.tag import TagSet
from pyasn1.type.univ import Integer, OctetString, Sequence, SequenceOf

NMAS_LDAP_EXT_VERSION: int

class Identity(OctetString):
    encoding: str

class LDAPDN(OctetString):
    tagSet: TagSet
    encoding: str

class Password(OctetString):
    tagSet: TagSet
    encoding: str

class LDAPOID(OctetString):
    tagSet: TagSet
    encoding: str

class GroupCookie(Integer):
    tagSet: TagSet

class NmasVer(Integer):
    tagSet: TagSet

class Error(Integer):
    tagSet: TagSet

class NmasGetUniversalPasswordRequestValue(Sequence):
    componentType: NamedTypes

class NmasGetUniversalPasswordResponseValue(Sequence):
    componentType: NamedTypes

class NmasSetUniversalPasswordRequestValue(Sequence):
    componentType: NamedTypes

class NmasSetUniversalPasswordResponseValue(Sequence):
    componentType: NamedTypes

class ReplicaList(SequenceOf):
    componentType: OctetString  # type: ignore[assignment]

class ReplicaInfoRequestValue(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class ReplicaInfoResponseValue(Sequence):
    tagSet: TagSet
    componentType: NamedTypes

class CreateGroupTypeRequestValue(Sequence):
    componentType: NamedTypes

class CreateGroupTypeResponseValue(Sequence):
    componentType: NamedTypes

class EndGroupTypeRequestValue(Sequence):
    componentType: NamedTypes

class EndGroupTypeResponseValue(Sequence):
    componentType: NamedTypes

class GroupingControlValue(Sequence):
    componentType: NamedTypes



############################################################################
# Original work Copyright 2018 Palantir Technologies, Inc.                 #
# Original work licensed under the MIT License.                            #
# See ThirdPartyNotices.txt in the project root for license information.   #
# All modifications Copyright (c) Open Law Library. All rights reserved.   #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################
import os
import sys

IS_WIN = os.name == "nt"
IS_PYODIDE = "pyodide" in sys.modules

pygls = "pygls"



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import Any

from serial.serialutil import *

def my_import(name: str) -> Any: ...  # Java object
def detect_java_comm(names: Iterable[str]) -> Any: ...  # Java object

comm: Any  # Java object

def device(portnumber: int) -> str: ...

class Serial(SerialBase):
    sPort: Any  # Java object
    def open(self) -> None: ...
    @property
    def in_waiting(self) -> int: ...
    def read(self, size: int = 1) -> bytes: ...
    def write(self, b: ReadableBuffer, /) -> int | None: ...
    def reset_input_buffer(self) -> None: ...
    def reset_output_buffer(self) -> None: ...
    @property
    def cts(self) -> bool: ...
    @property
    def dsr(self) -> bool: ...
    @property
    def ri(self) -> bool: ...
    @property
    def cd(self) -> bool: ...



from collections.abc import Sequence
from typing import Final
from typing_extensions import TypeAlias

from Xlib.display import Display
from Xlib.protocol import request, rq
from Xlib.xobject import drawable, resource

_RandRModeInfo13IntSequence: TypeAlias = Sequence[int]

extname: Final = "RANDR"
RRScreenChangeNotify: Final = 0
RRNotify: Final = 1
RRNotify_CrtcChange: Final = 0
RRNotify_OutputChange: Final = 1
RRNotify_OutputProperty: Final = 2
RRScreenChangeNotifyMask: Final = 0x1
RRCrtcChangeNotifyMask: Final = 0x2
RROutputChangeNotifyMask: Final = 0x4
RROutputPropertyNotifyMask: Final = 0x8
SetConfigSuccess: Final = 0
SetConfigInvalidConfigTime: Final = 1
SetConfigInvalidTime: Final = 2
SetConfigFailed: Final = 3
Rotate_0: Final = 1
Rotate_90: Final = 2
Rotate_180: Final = 4
Rotate_270: Final = 8
Reflect_X: Final = 16
Reflect_Y: Final = 32
HSyncPositive: Final = 0x00000001
HSyncNegative: Final = 0x00000002
VSyncPositive: Final = 0x00000004
VSyncNegative: Final = 0x00000008
Interlace: Final = 0x00000010
DoubleScan: Final = 0x00000020
CSync: Final = 0x00000040
CSyncPositive: Final = 0x00000080
CSyncNegative: Final = 0x00000100
HSkewPresent: Final = 0x00000200
BCast: Final = 0x00000400
PixelMultiplex: Final = 0x00000800
DoubleClock: Final = 0x00001000
ClockDivideBy2: Final = 0x00002000
Connected: Final = 0
Disconnected: Final = 1
UnknownConnection: Final = 2
PROPERTY_RANDR_EDID: Final = "EDID"
PROPERTY_SIGNAL_FORMAT: Final = "SignalFormat"
PROPERTY_SIGNAL_PROPERTIES: Final = "SignalProperties"
PROPERTY_CONNECTOR_TYPE: Final = "ConnectorType"
PROPERTY_CONNECTOR_NUMBER: Final = "ConnectorNumber"
PROPERTY_COMPATIBILITY_LIST: Final = "CompatibilityList"
PROPERTY_CLONE_LIST: Final = "CloneList"
SubPixelUnknown: Final = 0
SubPixelHorizontalRGB: Final = 1
SubPixelHorizontalBGR: Final = 2
SubPixelVerticalRGB: Final = 3
SubPixelVerticalBGR: Final = 4
SubPixelNone: Final = 5
BadRROutput: Final = 0
BadRRCrtc: Final = 1
BadRRMode: Final = 2

class BadRROutputError(Exception): ...
class BadRRCrtcError(Exception): ...
class BadRRModeError(Exception): ...

RandR_ScreenSizes: rq.Struct
RandR_ModeInfo: rq.Struct
RandR_Rates: rq.Struct
Render_Transform: rq.Struct
MonitorInfo: rq.Struct

class QueryVersion(rq.ReplyRequest): ...

def query_version(self: Display | resource.Resource) -> QueryVersion: ...

class _1_0SetScreenConfig(rq.ReplyRequest): ...
class SetScreenConfig(rq.ReplyRequest): ...

def set_screen_config(
    self: drawable.Drawable, size_id: int, rotation: int, config_timestamp: int, rate: int = 0, timestamp: int = 0
) -> SetScreenConfig: ...

class SelectInput(rq.Request): ...

def select_input(self: drawable.Window, mask: int) -> SelectInput: ...

class GetScreenInfo(rq.ReplyRequest): ...

def get_screen_info(self: drawable.Window) -> GetScreenInfo: ...

class GetScreenSizeRange(rq.ReplyRequest): ...

def get_screen_size_range(self: drawable.Window) -> GetScreenSizeRange: ...

class SetScreenSize(rq.Request): ...

def set_screen_size(
    self: drawable.Window,
    width: int,
    height: int,
    width_in_millimeters: int | None = None,
    height_in_millimeters: int | None = None,
) -> SetScreenSize: ...

class GetScreenResources(rq.ReplyRequest): ...

def get_screen_resources(self: drawable.Window) -> GetScreenResources: ...

class GetOutputInfo(rq.ReplyRequest): ...

def get_output_info(self: Display | resource.Resource, output: int, config_timestamp: int) -> GetOutputInfo: ...

class ListOutputProperties(rq.ReplyRequest): ...

def list_output_properties(self: Display | resource.Resource, output: int) -> ListOutputProperties: ...

class QueryOutputProperty(rq.ReplyRequest): ...

def query_output_property(self: Display | resource.Resource, output: int, property: int) -> QueryOutputProperty: ...

class ConfigureOutputProperty(rq.Request): ...

def configure_output_property(self: Display | resource.Resource, output: int, property: int) -> ConfigureOutputProperty: ...

class ChangeOutputProperty(rq.Request): ...

def change_output_property(
    self: Display | resource.Resource, output: int, property: int, type: int, mode: int, value: Sequence[float] | Sequence[str]
) -> ChangeOutputProperty: ...

class DeleteOutputProperty(rq.Request): ...

def delete_output_property(self: Display | resource.Resource, output: int, property: int) -> DeleteOutputProperty: ...

class GetOutputProperty(rq.ReplyRequest): ...

def get_output_property(
    self: Display | resource.Resource,
    output: int,
    property: int,
    type: int,
    long_offset: int,
    long_length: int,
    delete: bool = False,
    pending: bool = False,
) -> GetOutputProperty: ...

class CreateMode(rq.ReplyRequest): ...

def create_mode(self: drawable.Window, mode: _RandRModeInfo13IntSequence, name: str) -> CreateMode: ...

class DestroyMode(rq.Request): ...

def destroy_mode(self: Display | resource.Resource, mode: int) -> DestroyMode: ...

class AddOutputMode(rq.Request): ...

def add_output_mode(self: Display | resource.Resource, output: int, mode: int) -> AddOutputMode: ...

class DeleteOutputMode(rq.Request): ...

def delete_output_mode(self: Display | resource.Resource, output: int, mode: int) -> DeleteOutputMode: ...

class GetCrtcInfo(rq.ReplyRequest): ...

def get_crtc_info(self: Display | resource.Resource, crtc: int, config_timestamp: int) -> GetCrtcInfo: ...

class SetCrtcConfig(rq.ReplyRequest): ...

def set_crtc_config(
    self: Display | resource.Resource,
    crtc: int,
    config_timestamp: int,
    x: int,
    y: int,
    mode: int,
    rotation: int,
    outputs: Sequence[int],
    timestamp: int = 0,
) -> SetCrtcConfig: ...

class GetCrtcGammaSize(rq.ReplyRequest): ...

def get_crtc_gamma_size(self: Display | resource.Resource, crtc: int) -> GetCrtcGammaSize: ...

class GetCrtcGamma(rq.ReplyRequest): ...

def get_crtc_gamma(self: Display | resource.Resource, crtc: int) -> GetCrtcGamma: ...

class SetCrtcGamma(rq.Request): ...

def set_crtc_gamma(
    self: Display | resource.Resource, crtc: int, size: int, red: Sequence[int], green: Sequence[int], blue: Sequence[int]
) -> SetCrtcGamma: ...

class GetScreenResourcesCurrent(rq.ReplyRequest): ...

def get_screen_resources_current(self: drawable.Window) -> GetScreenResourcesCurrent: ...

class SetCrtcTransform(rq.Request): ...

def set_crtc_transform(self: Display | resource.Resource, crtc: int, n_bytes_filter: Sequence[int]) -> SetCrtcTransform: ...

class GetCrtcTransform(rq.ReplyRequest): ...

def get_crtc_transform(self: Display | resource.Resource, crtc: int) -> GetCrtcTransform: ...

class GetPanning(rq.ReplyRequest): ...

def get_panning(self: Display | resource.Resource, crtc: int) -> GetPanning: ...

class SetPanning(rq.ReplyRequest): ...

def set_panning(
    self: Display | resource.Resource,
    crtc: int,
    left: int,
    top: int,
    width: int,
    height: int,
    track_left: int,
    track_top: int,
    track_width: int,
    track_height: int,
    border_left: int,
    border_top: int,
    border_width: int,
    border_height: int,
    timestamp: int = 0,
) -> SetPanning: ...

class SetOutputPrimary(rq.Request): ...

def set_output_primary(self: drawable.Window, output: int) -> SetOutputPrimary: ...

class GetOutputPrimary(rq.ReplyRequest): ...

def get_output_primary(self: drawable.Window) -> GetOutputPrimary: ...

class GetMonitors(rq.ReplyRequest): ...

def get_monitors(self: drawable.Window, is_active: bool = True) -> GetMonitors: ...

class SetMonitor(rq.Request): ...

def set_monitor(
    self: drawable.Window, monitor_info: tuple[int, bool, bool, Sequence[int], int, int, int, int, int]
) -> SetMonitor: ...

class DeleteMonitor(rq.Request): ...

def delete_monitor(self: Display | resource.Resource, name: str) -> DeleteMonitor: ...

class ScreenChangeNotify(rq.Event): ...
class CrtcChangeNotify(rq.Event): ...
class OutputChangeNotify(rq.Event): ...
class OutputPropertyNotify(rq.Event): ...

def init(disp: Display, info: request.QueryExtension) -> None: ...
